"""NcatBot 的 Python 包管理工具。

本模块提供了管理 Python 包的工具，包括安装、卸载和依赖管理。
"""

import importlib
import importlib.metadata
import json
import subprocess
import sys
from typing import Any, Dict, List, Optional, Union

from packaging import version
from packaging.markers import UndefinedComparison
from packaging.requirements import Requirement
from packaging.specifiers import SpecifierSet

from ncatbot.cli.utils.constants import PYPI_SOURCE
from ncatbot.utils import get_log

logger = get_log("CLI")


class PipManagerException(Exception):
    """包管理器操作异常基类。

    当包管理操作失败时抛出，包含详细的错误信息。

    属性:
        message: 异常描述
        original_exception: 原始异常对象（如果有）
    """

    def __init__(self, message: str, original_exception: Optional[Exception] = None):
        super().__init__(message)
        self.original_exception = original_exception
        self.message = f"包管理器错误: {message}"


class PipTool:
    """Python 包管理核心类。

    提供全面的包管理功能，包括：
    - 包的安装/卸载
    - 包信息查询
    - 依赖管理
    - 环境验证
    - 批量操作

    属性:
        python_path (str): Python 解释器路径，默认为当前环境
        base_cmd (List[str]): 基础命令前缀
    """

    installed_packages = None

    def __init__(self, python_path: str = sys.executable):
        """初始化包管理器。

        Args:
            python_path: Python 解释器路径，默认为当前解释器
                        示例: "/usr/bin/python3"

        Raises:
            PipManagerException: 当基础依赖安装失败时
        """
        self.python_path = python_path
        self.base_cmd = [self.python_path, "-m"]

        # 通过自动安装必要的依赖来初始化
        try:
            # 我们可以假设这些已经安装好了
            pass
        except subprocess.CalledProcessError as exc:
            raise PipManagerException("安装基础依赖失败") from exc

    def _run_command(
        self, args: List[str], capture_output: bool = True, pip: bool = True
    ) -> subprocess.CompletedProcess:
        """执行系统命令（基础方法）。

        Args:
            args: 命令参数列表
            capture_output: 是否捕获输出，默认为 True
            pip: 是否在基础命令中添加 pip，默认为 True

        Returns:
            subprocess.CompletedProcess: 命令执行结果对象

        Raises:
            PipManagerException: 当命令执行失败时
        """
        full_cmd = self.base_cmd.copy()
        if pip:
            full_cmd.append("pip")
        full_cmd.extend(args)

        try:
            return subprocess.run(
                full_cmd,
                capture_output=capture_output,
                text=True,
                check=True,
                encoding="utf-8",
            )
        except subprocess.CalledProcessError as exc:
            error_msg = exc.stderr.strip() or exc.stdout.strip() or "未知错误"
            raise PipManagerException(
                f"命令执行失败: {' '.join(exc.cmd)}\n错误: {error_msg}",
                original_exception=exc,
            )

    def install(
        self,
        package: str,
        version: Optional[str] = None,
        upgrade: bool = False,
        no_deps: bool = False,
        index_url: Optional[str] = PYPI_SOURCE,
        extra_args: Optional[List[str]] = None,
    ) -> Dict[str, Any]:
        """安装 Python 包。

        Args:
            package: 包名
            version: 特定版本号，可选
            upgrade: 是否升级到最新版本，默认为 False
            no_deps: 是否跳过依赖安装，默认为 False
            index_url: 自定义 PyPI 镜像 URL，可选
            extra_args: 额外的 pip 参数，可选

        Returns:
            包含操作结果的字典:
            {
                "status": "success"|"failed",
                "package": 包名,
                "version": 安装的版本（可选）,
                "error": 错误信息（失败时存在）
            }

        示例:
            >>> pm = PipTool()
            >>> pm.install("requests", version="2.25.1", upgrade=True)
            {'status': 'success', 'package': 'requests==2.25.1'}
        """
        PipTool.installed_packages = None  # 清除缓存
        args = ["install"]
        if upgrade:
            args.append("--upgrade")
        if no_deps:
            args.append("--no-deps")
        if index_url:
            args.extend(["--index-url", index_url])
        if version:
            package = f"{package}=={version}"
        args.append(package)
        if extra_args:
            args.extend(extra_args)

        try:
            self._run_command(args, capture_output=False)
            return {"status": "success", "package": package}
        except PipManagerException as exc:
            return {
                "status": "failed",
                "package": package,
                "error": str(exc),
                "version": version or "latest",
            }

    def uninstall(self, package: str, confirm: bool = True) -> Dict[str, Any]:
        """卸载已安装的包。

        Args:
            package: 要卸载的包名
            confirm: 是否自动确认卸载，默认为 True

        Returns:
            包含操作结果的字典:
            {
                "status": "success"|"failed",
                "package": 包名,
                "error": 错误信息（失败时存在）
            }

        示例:
            >>> pm = PipTool()
            >>> pm.uninstall("requests")
            {'status': 'success', 'package': 'requests'}
        """
        args = ["uninstall", "-y"] if confirm else ["uninstall"]
        args.append(package)
        PipTool.installed_packages = None  # 清除缓存

        try:
            self._run_command(args)
            return {"status": "success", "package": package}
        except PipManagerException as exc:
            return {"status": "failed", "package": package, "error": str(exc)}

    def list_installed(self, format: str = "dict") -> Union[List[Dict], str, None]:
        """获取已安装包的列表。

        Args:
            format: 输出格式，支持 dict/json，默认为 dict

        Returns:
            包列表的格式化结果:
            - dict 格式: [{"name": str, "version": str}, ...]
            - json 格式: JSON 字符串
            失败时返回 None

        示例:
            >>> pm = PipTool()
            >>> pm.list_installed()
            [{'name': 'requests', 'version': '2.26.0'}, ...]
        """
        if PipTool.installed_packages:
            return PipTool.installed_packages
        try:
            result = self._run_command(["list", "--format=columns"])
            packages = []

            # 解析表格输出
            for line in result.stdout.strip().split("\n")[2:]:  # 跳过表头
                if not line.strip():
                    continue
                parts = line.split()
                packages.append(
                    {
                        "name": parts[0],
                        "version": parts[1],
                        "location": " ".join(parts[2:]) if len(parts) > 2 else "",
                    }
                )
            PipTool.installed_packages = self._format_output(packages, format)
            return PipTool.installed_packages
        except PipManagerException:
            return None

    def show_info(self, package: str, format: str = "dict") -> Union[Dict, str, None]:
        """获取包的详细信息。

        Args:
            package: 包名
            format: 输出格式，支持 dict/json，默认为 dict

        Returns:
            包信息的格式化结果:
            - dict 格式: 包含所有元数据的字典
            - json 格式: JSON 字符串
            如果包不存在则返回 None

        示例:
            >>> pm = PipTool()
            >>> pm.show_info("requests")
            {
                "name": "requests",
                "version": "2.26.0",
                "summary": "Python HTTP for Humans.",
                ...
            }
        """
        try:
            result = self._run_command(["show", package])
            info = {}

            for line in result.stdout.split("\n"):
                if ": " in line:
                    key, value = line.split(": ", 1)
                    info[key.strip().lower()] = value.strip()

            return self._format_output(info, format)
        except PipManagerException:
            return None

    def _format_output(self, data: Any, format: str) -> Any:
        """内部方法：格式化输出数据。

        Args:
            data: 要格式化的原始数据
            format: 目标格式（dict/json）

        Returns:
            格式化后的数据，如果格式不支持则返回原始数据
        """
        if format == "json":
            return json.dumps(data, indent=2, ensure_ascii=False)
        return data

    def generate_dependency_tree(self, root: Optional[str] = None) -> Dict:
        """生成包依赖树。

        Args:
            root: 指定根包名，默认为显示所有包的依赖

        Returns:
            依赖树的字典结构

        Raises:
            PipManagerException: 当依赖分析失败时

        示例:
            >>> pm = PipTool()
            >>> pm.generate_dependency_tree("requests")
            {
                "package": {
                    "key": "requests",
                    "package_name": "requests",
                    "installed_version": "2.32.3"
                    },
                "dependencies": [
                    {"package": "chardet", "version": "3.0.4"},
                    ...
                ]
            }
        """
        try:
            self._run_command(["install", "pipdeptree"])
            args = ["pipdeptree", "--json"]
            if root:
                args.extend(["-p", root])

            output = self._run_command(args, pip=False).stdout
            return json.loads(output)
        except json.JSONDecodeError as exc:
            raise PipManagerException("依赖树解析失败") from exc
        except Exception as exc:
            raise PipManagerException("依赖分析失败") from exc

    def verify_environment(self) -> Dict[str, List]:
        """验证当前环境依赖兼容性（优化版本）。

        Returns:
            包含冲突的字典:
            {
                "conflicts": [
                    {
                        "package": 包名,
                        "dependency": 依赖名,
                        "required": 所需版本,
                        "installed": 已安装版本
                    },
                    ...
                ]
            }

        示例:
            >>> pm = PipTool()
            >>> pm.verify_environment()
            {'conflicts': [
                {
                    'package': 'package1',
                    'dependency': 'dependency1',
                    'required': '>=1.0.0',
                    'installed': '0.9.0'
                }
            ]}
        """
        conflicts = []
        installed = {}

        # 获取所有已安装的包及其名称和版本（小写标准化）
        for dist in importlib.metadata.distributions():
            name = dist.metadata.get("Name", "").lower()
            if name:  # 跳过没有名称的包
                installed[name] = dist.version

        # 检查每个包的依赖关系
        for dist in importlib.metadata.distributions():
            package_name = dist.metadata.get("Name", "").lower()
            if not package_name:
                continue

            try:
                requirements = importlib.metadata.requires(package_name) or []
            except Exception:
                continue

            for req_str in requirements:
                try:
                    req = Requirement(req_str)
                except Exception:
                    continue  # 跳过依赖解析失败的包

                # 处理环境标记
                if req.marker:
                    try:
                        if not req.marker.evaluate():
                            continue
                    except UndefinedComparison:
                        continue  # 跳过包含未定义变量的条件
                    except Exception:
                        continue  # 跳过其他异常

                dep_name = req.name.lower()
                installed_version = installed.get(dep_name)
                required_version = req.specifier

                # 如果没有版本要求则跳过
                if not required_version:
                    continue

                # 依赖未安装
                if installed_version is None:
                    conflicts.append(
                        {
                            "package": package_name,
                            "dependency": dep_name,
                            "required": str(req.specifier),
                            "installed": None,
                        }
                    )
                    continue

                # 版本不满足要求
                if not req.specifier.contains(installed_version, prereleases=True):
                    conflicts.append(
                        {
                            "package": package_name,
                            "dependency": dep_name,
                            "required": str(req.specifier),
                            "installed": installed_version,
                        }
                    )

        return {"conflicts": conflicts}

    def _parse_requirements(self, package: str) -> List[Requirement]:
        """解析包依赖要求（内部方法）。"""
        info = self.show_info(package)
        if not info or "requires" not in info:
            return []
        return [Requirement(r) for r in info["requires"].split(", ") if r]

    def _check_requirement(self, requirement: Requirement) -> bool:
        """检查依赖是否满足（内部方法）。"""
        installed_info = self.show_info(requirement.name)
        if not installed_info:
            return False
        return requirement.specifier.contains(installed_info["version"])

    def compare_versions(self, installed_version: str, required_version: str) -> bool:
        """比较版本以检查是否满足要求。

        Args:
            installed_version: 已安装的版本号
            required_version: 所需的版本规范

        Returns:
            bool: 如果已安装版本满足要求则为 True
        """
        try:
            # 如果是精确版本比较（无运算符）
            if required_version.isdigit() or all(
                part.isdigit() for part in required_version.split(".")
            ):
                return installed_version == required_version

            if any(op in required_version for op in ["==", ">=", "<=", ">", "<", "~="]):
                # 创建版本规范
                spec = SpecifierSet(required_version)
                # 检查已安装版本是否满足规范
                return version.parse(installed_version) in spec

            # 默认情况，假设需要精确匹配
            return installed_version == required_version
        except Exception:
            return False


def install_pip_dependencies(path: str) -> bool:
    """从 requirements.txt 文件安装依赖。"""
    pm = PipTool()
    with open(path, "r", encoding="utf-8") as f:
        for line in f:
            line = line.strip()
            if not line or line.startswith("#"):
                continue
            pm.install(line)


if __name__ == "__main__":

    def main():
        """示例用法"""
        pm = PipTool()

        # # 安装包
        # print(pm.install("requests"))

        # # 列出已安装包
        # print(pm.list_installed(format="json"))

        # # 显示包信息
        # print(pm.show_info("requests"))

        # # 生成依赖树
        # print()
        # print(json.dumps(pm.generate_dependency_tree("requests")))

        # 环境验证
        print(pm.verify_environment())

    main()



# from ncatbot.plugin_system.loader import PluginLoader
from typing import Callable, Optional, List, Tuple, Dict, Any, Union, final
from datetime import datetime
import re
import time
import asyncio
import functools
import threading
from schedule import Scheduler
from ncatbot.utils import run_coroutine


class TimeTaskScheduler:
    """
    任务调度器，支持以下特性:
    - 多模式调度: 间隔任务/每日定点任务/一次性任务
    - 动态参数生成函数/预定义静态参数传入
    - 外部循环单步执行模式/独立执行模式
    - 执行条件判断
    - 运行次数限制

    Attributes:
        _scheduler (Scheduler): 内部调度器实例
        _jobs (list): 存储所有任务信息的列表
    """

    def __init__(self):
        self._scheduler = Scheduler()
        self._jobs = []
        threading.Thread(target=self.loop, daemon=True).start()

    def loop(self):
        while True:
            self.step()
            time.sleep(0.2)

    def _parse_time(self, time_str: str) -> dict:
        """
        解析时间参数为调度配置字典，支持格式:
        - 一次性任务: 'YYYY-MM-DD HH:MM:SS' 或 GitHub Action格式 'YYYY:MM:DD-HH:MM:SS'
        - 每日任务: 'HH:MM'
        - 间隔任务:
            * 基础单位: '120s', '2h30m', '0.5d'
            * 冒号分隔: '00:15:30' (时:分:秒)
            * 自然语言: '2天3小时5秒'

        Args:
            time_str (str): 时间参数字符串

        Returns:
            dict: 调度配置字典，包含以下键:
                - type: 调度类型 ('once'/'daily'/'interval')
                - value: 具体参数 (秒数/时间字符串)

        Raises:
            ValueError: 当时间格式无效时抛出
        """
        # 尝试解析为一次性任务
        try:
            if re.match(r"^\d{4}:\d{2}:\d{2}-\d{2}:\d{2}:\d{2}$", time_str):
                dt_str = time_str.replace(":", "-", 2).replace("-", " ", 1)
                dt = datetime.strptime(dt_str, "%Y-%m-%d %H:%M:%S")
            else:
                dt = datetime.strptime(time_str, "%Y-%m-%d %H:%M:%S")

            now = datetime.now()
            if dt < now:
                raise ValueError("指定的时间已过期")

            return {"type": "once", "value": (dt - now).total_seconds()}
        except ValueError:
            pass

        # 尝试解析为每日任务
        if re.match(r"^([0-1][0-9]|2[0-3]):([0-5][0-9])$", time_str):
            try:
                datetime.strptime(time_str, "%H:%M")
                return {"type": "daily", "value": time_str}
            except ValueError:
                pass

        # 解析为间隔任务
        try:
            return {"type": "interval", "value": self._parse_interval(time_str)}
        except ValueError as e:
            raise ValueError(f"无效的时间格式: {time_str}") from e

    def _parse_interval(self, time_str: str) -> int:
        """
        解析间隔时间参数为秒数

        Args:
            time_str (str): 间隔时间字符串，支持格式:
                * 基础单位: '120s', '2h30m', '0.5d'
                * 冒号分隔: '00:15:30' (时:分:秒)
                * 自然语言: '2天3小时5秒'

        Returns:
            int: 总秒数

        Raises:
            ValueError: 当格式无效时抛出
        """
        units = {"d": 86400, "h": 3600, "m": 60, "s": 1}

        # 单位组合格式 (如2h30m)
        unit_match = re.match(r"^([\d.]+)([dhms])?$", time_str, re.IGNORECASE)
        if unit_match:
            num, unit = unit_match.groups()
            unit = unit.lower() if unit else "s"
            return int(float(num) * units[unit])

        # 冒号分隔格式 (如01:30:00)
        if ":" in time_str:
            parts = list(map(float, time_str.split(":")))
            multipliers = [1, 60, 3600, 86400][-len(parts) :]
            return int(sum(p * m for p, m in zip(parts[::-1], multipliers)))

        # 自然语言格式 (如2天3小时5秒)
        lang_match = re.match(r"(\d+)\s*天\s*(\d+)\s*小时\s*(\d+)\s*秒", time_str)
        if lang_match:
            d, h, s = map(int, lang_match.groups())
            return d * 86400 + h * 3600 + s

        raise ValueError("无法识别的间隔时间格式")

    def add_job(
        self,
        job_func: Callable,
        name: str,
        interval: Union[str, int, float],
        conditions: Optional[List[Callable[[], bool]]] = None,
        max_runs: Optional[int] = None,
        args: Optional[Tuple] = None,
        kwargs: Optional[Dict] = None,
        args_provider: Optional[Callable[[], Tuple]] = None,
        kwargs_provider: Optional[Callable[[], Dict[str, Any]]] = None,
    ) -> bool:
        """
        添加定时任务

        Args:
            job_func (Callable): 要执行的任务函数
            name (str): 任务唯一标识名称
            interval (Union[str, int, float]): 调度时间参数
            conditions (Optional[List[Callable]]): 执行条件列表
            max_runs (Optional[int]): 最大执行次数
            args (Optional[Tuple]): 静态位置参数
            kwargs (Optional[Dict]): 静态关键字参数
            args_provider (Optional[Callable]): 动态位置参数生成函数
            kwargs_provider (Optional[Callable]): 动态关键字参数生成函数

        Returns:
            bool: 是否添加成功

        Raises:
            ValueError: 当参数冲突或时间格式无效时
        """
        # 名称唯一性检查
        if any(job["name"] == name for job in self._jobs):
            print(f"任务添加失败: 名称 '{name}' 已存在")
            return False

        # 参数冲突检查
        if (args and args_provider) or (kwargs and kwargs_provider):
            raise ValueError("静态参数和动态参数生成器不能同时使用")

        try:
            # 解析时间参数
            interval_cfg = self._parse_time(str(interval))

            # 一次性任务强制设置max_runs=1
            if interval_cfg["type"] == "once":
                if max_runs and max_runs != 1:
                    raise ValueError("一次性任务必须设置 max_runs=1")
                max_runs = 1

            job_info = {
                "name": name,
                "func": job_func,
                "max_runs": max_runs,
                "run_count": 0,
                "conditions": conditions or [],
                "static_args": args,
                "static_kwargs": kwargs or {},
                "args_provider": args_provider,
                "kwargs_provider": kwargs_provider,
            }

            @functools.wraps(job_func)
            def job_wrapper():
                # 执行次数检查
                if (
                    job_info["max_runs"]
                    and job_info["run_count"] >= job_info["max_runs"]
                ):
                    self.remove_job(name)
                    return

                # 条件检查
                if not all(cond() for cond in job_info["conditions"]):
                    return

                # 参数处理
                dyn_args = (
                    job_info["args_provider"]() if job_info["args_provider"] else ()
                )
                dyn_kwargs = (
                    job_info["kwargs_provider"]() if job_info["kwargs_provider"] else {}
                )
                final_args = dyn_args or job_info["static_args"] or ()
                final_kwargs = {**job_info["static_kwargs"], **dyn_kwargs}

                # 执行任务
                try:
                    if asyncio.iscoroutinefunction(job_info["func"]):
                        run_coroutine(job_info["func"], *final_args, **final_kwargs)
                    else:
                        job_info["func"](*final_args, **final_kwargs)
                    job_info["run_count"] += 1
                except Exception as e:
                    print(f"任务执行失败 [{name}]: {str(e)}")

            # 创建调度任务
            if interval_cfg["type"] == "interval":
                job = self._scheduler.every(interval_cfg["value"]).seconds.do(
                    job_wrapper
                )
            elif interval_cfg["type"] == "daily":
                job = (
                    self._scheduler.every()
                    .day.at(interval_cfg["value"])
                    .do(job_wrapper)
                )
            elif interval_cfg["type"] == "once":
                job = self._scheduler.every(interval_cfg["value"]).seconds.do(
                    job_wrapper
                )

            job_info["schedule_job"] = job
            self._jobs.append(job_info)
            return True

        except Exception as e:
            print(f"任务添加失败: {str(e)}")
            return False

    def step(self) -> None:
        """单步执行"""
        self._scheduler.run_pending()

    def run(self) -> None:
        """独立运行"""
        try:
            while True:
                self.step()
                # 计算下一次任务的最早执行时间
                next_run_time = None
                for job in self._jobs:
                    if job["schedule_job"].next_run is not None:
                        if (
                            next_run_time is None
                            or job["schedule_job"].next_run < next_run_time
                        ):
                            next_run_time = job["schedule_job"].next_run
                # 如果有任务待执行，计算需要等待的时间
                if next_run_time is not None:
                    sleep_time = (next_run_time - datetime.now()).total_seconds()
                    if sleep_time > 0:
                        time.sleep(sleep_time)
                    else:
                        # 如果已经过了下一次执行时间，立即检查任务
                        continue
                else:
                    # 如果没有任务待执行，适当等待
                    time.sleep(0.2)  # 我觉得吧, 应该不需要等待
        except KeyboardInterrupt:
            print("\n调度器已安全停止")

    def remove_job(self, name: str) -> bool:
        """
        移除指定名称的任务

        Args:
            name (str): 要移除的任务名称

        Returns:
            bool: 是否成功找到并移除任务
        """
        for job in self._jobs:
            if job["name"] == name:
                self._scheduler.cancel_job(job["schedule_job"])
                self._jobs.remove(job)
                return True
        return False

    def get_job_status(self, name: str) -> Optional[dict]:
        """
        获取任务状态信息

        Args:
            name (str): 任务名称

        Returns:
            Optional[dict]: 包含状态信息的字典，格式:
                {
                    'name': 任务名称,
                    'next_run': 下次运行时间,
                    'run_count': 已执行次数,
                    'max_runs': 最大允许次数
                }
        """
        for job in self._jobs:
            if job["name"] == name:
                return {
                    "name": name,
                    "next_run": job["schedule_job"].next_run,
                    "run_count": job["run_count"],
                    "max_runs": job["max_runs"],
                }
        return None


class TimeTaskMixin:
    """定时任务调度混入类，提供定时任务的管理功能。

    # 描述
    该混入类提供了定时任务的添加、移除等管理功能。支持灵活的任务调度配置，
    包括固定间隔执行、条件触发、参数动态生成等特性。

    # 属性
    - `_time_task_scheduler` (TimeTaskScheduler): 时间任务调度器实例

    # 特性
    - 支持固定时间间隔的任务调度
    - 支持条件触发机制
    - 支持最大执行次数限制
    - 支持动态参数生成
    """

    _time_task_scheduler: TimeTaskScheduler = TimeTaskScheduler()

    # TODO 防止某些任务导致调度器饿死(超时机制)
    @final
    def add_scheduled_task(
        self,
        job_func: Callable,
        name: str,
        interval: Union[str, int, float],
        conditions: Optional[List[Callable[[], bool]]] = None,
        max_runs: Optional[int] = None,
        args: Optional[Tuple] = None,
        kwargs: Optional[Dict] = None,
        args_provider: Optional[Callable[[], Tuple]] = None,
        kwargs_provider: Optional[Callable[[], Dict[str, Any]]] = None,
    ) -> bool:
        """添加一个定时任务。

        Args:
            job_func (Callable): 要执行的任务函数。
            name (str): 任务名称。
            interval (Union[str, int, float]): 任务执行的时间间隔。
            conditions (Optional[List[Callable[[], bool]]], optional): 任务执行的条件列表。默认为None。
            max_runs (Optional[int], optional): 任务的最大执行次数。默认为None。
            args (Optional[Tuple], optional): 任务函数的位置参数。默认为None。
            kwargs (Optional[Dict], optional): 任务函数的关键字参数。默认为None。
            args_provider (Optional[Callable[[], Tuple]], optional): 提供任务函数位置参数的函数。默认为None。
            kwargs_provider (Optional[Callable[[], Dict[str, Any]]], optional): 提供任务函数关键字参数的函数。默认为None。

        Returns:
            bool: 如果任务添加成功返回True，否则返回False。
        """
        job_info = {
            "name": name,
            "job_func": job_func,
            "interval": interval,
            "max_runs": max_runs,
            "conditions": conditions or [],
            "args": args,
            "kwargs": kwargs or {},
            "args_provider": args_provider,
            "kwargs_provider": kwargs_provider,
        }
        return self._time_task_scheduler.add_job(**job_info)

    @final
    def remove_scheduled_task(self, task_name: str):
        """移除一个定时任务。

        Args:
            task_name (str): 要移除的任务名称。

        Returns:
            bool: 如果任务移除成功返回True，否则返回False。
        """
        return self._time_task_scheduler.remove_job(name=task_name)

    @final
    def restart_scheduler(self) -> None:
        """重启定时任务调度器。"""
        pass



"""过滤器装饰器 v2.0"""

from typing import Callable, Union, TYPE_CHECKING
from .builtin import GroupFilter, PrivateFilter, AdminFilter, RootFilter, TrueFilter
from .base import BaseFilter

if TYPE_CHECKING:
    from .base import BaseFilter


def filter(*filters: Union[str, "BaseFilter"]):
    from .registry import filter_registry

    """为函数添加过滤器的装饰器

    Usage:
        @filter("my_filter")
        @filter(GroupFilter())
        @filter(GroupFilter(), AdminFilter())
        def my_command(event):
            pass
    """

    def decorator(func: Callable) -> Callable:
        filter_registry.add_filter_to_function(func, *filters)
        return func

    return decorator


def group_filter(func: Callable) -> Callable:
    """群聊专用装饰器"""
    deco = filter(GroupFilter())
    return deco(func)


def private_filter(func: Callable) -> Callable:
    """私聊专用装饰器"""
    deco = filter(PrivateFilter())
    return deco(func)


def admin_filter(func: Callable) -> Callable:
    """管理员专用装饰器"""
    deco = filter(AdminFilter())
    return deco(func)


def root_filter(func: Callable) -> Callable:
    """Root专用装饰器"""
    deco = filter(RootFilter())
    return deco(func)


def on_message(func: Callable) -> Callable:
    """消息专用装饰器"""
    deco = filter(TrueFilter())
    return deco(func)


# 组合装饰器
def admin_group_filter(func: Callable) -> Callable:
    """管理员群聊专用装饰器"""
    deco = filter(GroupFilter(), AdminFilter())
    return deco(func)


def admin_private_filter(func: Callable) -> Callable:
    """管理员私聊专用装饰器"""
    deco = filter(PrivateFilter(), AdminFilter())
    return deco(func)


def on_request(func: Callable) -> Callable:
    """请求专用装饰器"""
    from ..legacy_registry import legacy_registry

    legacy_registry._request_event.append(func)
    return func


def on_notice(func: Callable) -> Callable:
    """通知专用装饰器"""
    from ..legacy_registry import legacy_registry

    legacy_registry._notice_event.append(func)
    return func


# 兼容
admin_only = admin_filter
root_only = root_filter
private_only = private_filter
group_only = group_filter



import asyncio
import re
import threading
import uuid
import time
import ctypes
import queue
from queue import Queue
import traceback
from concurrent.futures import Future
from functools import lru_cache
from ncatbot.utils import get_log

# from logging import getLogger
from typing import Any, Callable, Dict, List, Optional, Tuple, TYPE_CHECKING, Set

from ncatbot.utils import run_coroutine

if TYPE_CHECKING:
    from ..base_plugin import BasePlugin

from .event import NcatBotEvent

LOG = get_log("EventBus")


class HandlerTimeoutError(Exception):
    def __init__(self, meta_data, handler, time):
        super().__init__()
        self.meta_data = meta_data
        self.handler = handler
        self.time = time

    meta_data: dict
    handler: str
    time: float

    def __str__(self):
        return f"来自 {self.meta_data['name']} 的处理器 {self.handler} 执行超时 {self.time}"


class EventBus:
    def __init__(self, default_timeout: float = 5, max_workers: int = 1) -> None:
        """
        事件总线实现 - 线程池

        Args:
            default_timeout: 默认处理器超时时间（秒）
            max_workers: 最大工作线程数
        """
        self._exact: Dict[str, List[Tuple]] = {}
        self._regex: List[Tuple] = []
        self._lock = threading.Lock()
        self.default_timeout = default_timeout

        # 线程池状态
        self.task_queue = queue.Queue()
        self.workers: Set[threading.Thread] = set()
        self.worker_lock = threading.Lock()
        self.worker_timeouts: Dict[int, float] = {}  # {thread_id: timeout}
        self.active_tasks: Dict[int, Future] = {}  # {thread_id: future}

        # 存储处理器元数据
        self._handler_meta: Dict[uuid.UUID, Dict] = {}
        self.result_queues: Dict[uuid.UUID, asyncio.Queue] = {}  # 添加结果队列字典

        # 初始化工作线程
        for _ in range(max_workers):
            self._add_worker()

        # 启动监控线程
        self.monitor_thread = threading.Thread(
            target=self._monitor_timeouts, daemon=True
        )
        self.monitor_thread.start()

    def _add_worker(self):
        """添加新的工作线程"""
        worker = threading.Thread(
            target=self._worker_loop, daemon=True, name="EventBus_Worker"
        )
        worker.start()
        self.workers.add(worker)

    def _worker_loop(self):
        """工作线程主循环"""
        thread_id = threading.get_ident()
        while True:
            try:
                # 获取任务
                task = self.task_queue.get(timeout=0.1)
                runner, handler, event, timeout, result_queue, hid = task
                # LOG.debug(f"线程执行任务: {handler.__name__}")
                result_queue: Queue

                # 记录任务开始时间
                start_time = time.time()
                self.worker_timeouts[thread_id] = start_time + timeout

                try:
                    # 执行任务
                    result = runner(handler, event)
                    result_queue.put(result)
                    # LOG.debug(f"任务结果: {id(result_queue)}")
                except Exception as e:
                    result_queue.put(e)
                finally:
                    # 清理任务状态
                    del self.worker_timeouts[thread_id]
                    self.task_queue.task_done()
                    # LOG.debug(f"线程任务结束: {handler.__name__}")
            except queue.Empty:
                # 检查是否需要退出
                if len(self.workers) > max(5, len(self.workers) * 0.8):
                    break

    def _monitor_timeouts(self):
        """监控超时任务并强制终止线程"""
        while True:
            time.sleep(self.default_timeout / 4)
            current_time = time.time()

            with self.worker_lock:
                # 检查所有工作线程
                for worker in list(self.workers):
                    thread_id = worker.ident
                    if thread_id in self.worker_timeouts:
                        # 检查是否超时
                        if current_time > self.worker_timeouts[thread_id]:
                            # 强制终止超时线程
                            self._terminate_thread(worker)
                            self.workers.remove(worker)

                            # 补充新线程
                            self._add_worker()

                            # 设置future异常
                            if thread_id in self.active_tasks:
                                future = self.active_tasks[thread_id]
                                if not future.done():
                                    future.set_exception(TimeoutError("处理器超时"))
                                del self.active_tasks[thread_id]

    def _terminate_thread(self, thread: threading.Thread):
        """强制终止线程"""
        if not thread.is_alive():
            return

        exc = ctypes.py_object(TimeoutError)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(
            ctypes.c_long(thread.ident), exc
        )

        if res == 0:
            raise ValueError("无效的线程 ID")
        elif res != 1:
            ctypes.pythonapi.PyThreadState_SetAsyncExc(thread.ident, None)

    def _run_handler(self, handler: Callable, event: NcatBotEvent) -> Any:
        """
        执行处理程序
        """
        # 如果是异步函数，在独立事件循环中运行
        # LOG.debug(f"执行处理程序: {handler.__name__}")
        try:
            if asyncio.iscoroutinefunction(handler):
                return run_coroutine(handler, event)
            else:
                return handler(event)
        except Exception as e:
            LOG.error(f"执行处理程序 {handler.__name__} 时发生错误: {e}")
            LOG.info(f"错误堆栈: {traceback.format_exc()}")
            raise e

    def subscribe(
        self,
        event_type: str,
        handler: Callable[[NcatBotEvent], Any],
        priority: int = 0,
        timeout: Optional[float] = None,
        plugin: Optional["BasePlugin"] = None,
    ) -> uuid.UUID:
        """订阅事件处理程序"""
        hid = uuid.uuid4()
        timeout_val = timeout if timeout is not None else self.default_timeout

        # 记录处理器元数据
        if plugin:
            self._handler_meta[hid] = plugin.meta_data

        with self._lock:
            if event_type.startswith("re:"):
                pattern = _compile_regex(event_type[3:])
                self._regex.append((pattern, priority, handler, hid, timeout_val))
                self._regex.sort(key=lambda t: (-t[1], t[2].__name__))
            else:
                bucket = self._exact.setdefault(event_type, [])
                bucket.append((None, priority, handler, hid, timeout_val))
                bucket.sort(key=lambda t: (-t[1], t[2].__name__))
        return hid

    def unsubscribe(self, handler_id: uuid.UUID) -> bool:
        """取消订阅事件处理程序"""
        removed = False
        with self._lock:
            # 删除元数据
            if handler_id in self._handler_meta:
                del self._handler_meta[handler_id]

            # 处理精确匹配
            for typ in list(self._exact.keys()):
                original_len = len(self._exact[typ])
                self._exact[typ] = [h for h in self._exact[typ] if h[3] != handler_id]
                removed |= len(self._exact[typ]) != original_len
                if not self._exact[typ]:
                    del self._exact[typ]

            # 处理正则匹配
            original_len = len(self._regex)
            self._regex = [h for h in self._regex if h[3] != handler_id]
            removed |= len(self._regex) != original_len
        return removed

    async def publish(self, event: NcatBotEvent) -> List[Any]:
        """发布事件并等待所有处理器完成"""
        handlers = self._collect_handlers(event.type)
        handler_meta = self._handler_meta.copy()
        result_queues = {}

        # 为每个handler提交任务
        for _, _, handler, hid, timeout in handlers:
            if event._propagation_stopped:
                break

            # 为每个任务创建结果队列
            result_queue = Queue()
            self.result_queues[hid] = result_queue
            result_queues[hid] = result_queue

            # 提交任务到线程池
            LOG.debug(f"提交任务到线程池: {handler.__name__}, {hid}")
            self.task_queue.put(
                (self._run_handler, handler, event, timeout, result_queue, hid)
            )
            try:
                result = result_queue.get(timeout=timeout)
                if isinstance(result, Exception):
                    LOG.error(
                        f"任务 {handler_meta[hid].__name__} {hid} 发生错误: {result}"
                    )
                    if isinstance(result, TimeoutError):
                        event.add_exception(
                            HandlerTimeoutError(
                                meta_data=handler_meta[hid], handler=hid, time=timeout
                            )
                        )
                    else:
                        event.add_exception(result)
                else:
                    event._results.append(result)
            except asyncio.TimeoutError:
                LOG.error(f"任务 {hid} 超时")
                event.add_exception(
                    HandlerTimeoutError(
                        meta_data=handler_meta[hid], handler=hid, time=timeout
                    )
                )
            finally:
                # 清理结果队列
                try:
                    del self.result_queues[hid]
                except Exception as e:
                    LOG.error(f"清理结果队列时发生错误: {e}")
                    LOG.info(traceback.format_exc())

        try:
            for e in event.exceptions:
                LOG.error(str(e))
            # LOG.debug(f"收集结果: {event._results}")
            return event._results.copy()
        except Exception as e:
            LOG.error(f"收集结果时发生错误: {e}")
            LOG.info(traceback.format_exc())
            return []

    def _collect_handlers(self, event_type: str) -> List[Tuple]:
        """收集匹配的事件处理程序"""
        with self._lock:
            # 获取精确匹配的处理程序
            exact_handlers = self._exact.get(event_type, [])[:]

            # 获取正则匹配的处理程序
            regex_handlers = []
            for pattern, priority, handler, hid, timeout in self._regex:
                if pattern and pattern.match(event_type):
                    regex_handlers.append((pattern, priority, handler, hid, timeout))

        # 合并并排序处理程序
        all_handlers = exact_handlers + regex_handlers
        all_handlers.sort(key=lambda t: (-t[1], t[2].__name__))
        return all_handlers

    def shutdown(self):
        """关闭事件总线并清理资源"""
        # 终止所有工作线程
        with self.worker_lock:
            for worker in self.workers:
                if worker.is_alive():
                    self._terminate_thread(worker)
            self.workers.clear()

            # 清空任务队列
            while not self.task_queue.empty():
                try:
                    self.task_queue.get_nowait()
                    self.task_queue.task_done()
                except queue.Empty:
                    break


@lru_cache(maxsize=128)
def _compile_regex(pattern: str) -> re.Pattern:
    """编译正则表达式并缓存"""
    try:
        return re.compile(pattern)
    except re.error as e:
        raise ValueError(f"无效正则表达式: {pattern}") from e



"""NcatBot 工具包。"""

from ncatbot.utils.config import ncatbot_config
from ncatbot.utils.config import ncatbot_config as config
from ncatbot.utils.logger import get_log
from ncatbot.utils.status import Status, status
from ncatbot.utils.network_io import gen_url_with_proxy, get_json, post_json
from ncatbot.utils.error import NcatBotError, NcatBotValueError, NcatBotConnectionError
from ncatbot.utils.thread_pool import run_coroutine, ThreadPool

# Re-export assets
from ncatbot.utils.assets import (
    Color,
    NAPCAT_WEBUI_SALT,
    WINDOWS_NAPCAT_DIR,
    LINUX_NAPCAT_DIR,
    INSTALL_SCRIPT_URL,
    NAPCAT_CLI_URL,
    PYPI_URL,
    REQUEST_SUCCESS,
    OFFICIAL_GROUP_MESSAGE_EVENT,
    OFFICIAL_PRIVATE_MESSAGE_EVENT,
    OFFICIAL_REQUEST_EVENT,
    OFFICIAL_NOTICE_EVENT,
    OFFICIAL_STARTUP_EVENT,
    OFFICIAL_SHUTDOWN_EVENT,
    OFFICIAL_HEARTBEAT_EVENT,
    PLUGIN_BROKEN_MARK,
    STATUS_ONLINE,
    STATUS_Q_ME,
    STATUS_LEAVE,
    STATUS_BUSY,
    STATUS_DND,
    STATUS_HIDDEN,
    STATUS_LISTENING,
    STATUS_LOVE_YOU,
    STATUS_LEARNING,
    PermissionGroup,
    DefaultPermission,
    EVENT_QUEUE_MAX_SIZE,
    PLUGINS_DIR,
    META_CONFIG_PATH,
    PERSISTENT_DIR,
)

__all__ = [
    # 配置导出
    "ncatbot_config",
    "config",
    # 日志导出
    "get_log",
    # 状态导出
    "status",
    "Status",
    # 网络工具
    "gen_url_with_proxy",
    "post_json",
    "get_json",
    # 错误处理
    "NcatBotError",
    "NcatBotValueError",
    "NcatBotConnectionError",
    # 线程池
    "ThreadPool",
    "run_coroutine",
    # 资源/常量
    "Color",
    "NAPCAT_WEBUI_SALT",
    "WINDOWS_NAPCAT_DIR",
    "LINUX_NAPCAT_DIR",
    "INSTALL_SCRIPT_URL",
    "NAPCAT_CLI_URL",
    "PYPI_URL",
    "REQUEST_SUCCESS",
    "OFFICIAL_GROUP_MESSAGE_EVENT",
    "OFFICIAL_PRIVATE_MESSAGE_EVENT",
    "OFFICIAL_REQUEST_EVENT",
    "OFFICIAL_NOTICE_EVENT",
    "OFFICIAL_STARTUP_EVENT",
    "OFFICIAL_SHUTDOWN_EVENT",
    "OFFICIAL_HEARTBEAT_EVENT",
    "PLUGIN_BROKEN_MARK",
    "STATUS_ONLINE",
    "STATUS_Q_ME",
    "STATUS_LEAVE",
    "STATUS_BUSY",
    "STATUS_DND",
    "STATUS_HIDDEN",
    "STATUS_LISTENING",
    "STATUS_LOVE_YOU",
    "STATUS_LEARNING",
    "StatusConstants",
    "PermissionGroup",
    "DefaultPermission",
    "EVENT_QUEUE_MAX_SIZE",
    "PLUGINS_DIR",
    "META_CONFIG_PATH",
    "PERSISTENT_DIR",
]



from .func_mixin import FunctionMixin, Filter, Func
from typing import Callable, Any, List
from ncatbot.core.event import BaseMessageEvent
from ncatbot.utils import PermissionGroup
import asyncio
import functools


class AliasFilter(Filter):
    def __init__(self, name: str, aliases: List[str]):
        self.name = name
        self.aliases = aliases

    def check(self, event: BaseMessageEvent) -> bool:
        if event.raw_message.startswith("/" + self.name):
            return True
        if self.aliases is not None:
            for aliases in self.aliases:
                if event.raw_message.startswith("/" + aliases):
                    return True
        return False


class Command:
    def __init__(self, data: dict):
        self.name = data["name"]
        self.aliases = data["aliases"]
        self.permission = data["permission"]
        self.description = data["description"]
        self.usage = data["usage"]
        self.examples = data["examples"]
        self.func: Func = data["func"]

    def __str__(self) -> str:
        return f"{self.name} - {self.description}"

    def __repr__(self) -> str:
        return f"Command(name={self.name}, permission={self.permission}, description={self.description}, usage={self.usage}, examples={self.examples}, func={self.func})"

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, Command):
            return self.name == other.name
        if isinstance(other, str):
            return self.name == other
        return False


class CommandMixin(FunctionMixin):
    _registered_command_activators: List[str] = []

    def unregister_command(self, name: str):
        for command in self._registered_commands:
            if command.name == name:
                self.unregister_func(command.func.name)
                self._registered_commands.remove(command)
                break
        else:
            raise ValueError(f"命令 {name} 不存在")

    def get_registered_commands(self) -> List[Command]:
        if not hasattr(self, "_registered_commands"):
            self._registered_commands = []
        return self._registered_commands

    def _create_command_handler(self, handler: Callable) -> Callable:
        @functools.wraps(handler)
        async def warpped_handler(event: BaseMessageEvent, handler=handler):
            args = event.raw_message.split()
            if asyncio.iscoroutinefunction(handler):
                return await handler(event, args)
            else:
                return handler(event, args)

        return warpped_handler

    def register_command(
        self,
        name: str,
        handler: Callable[[BaseMessageEvent, List[str]], Any],
        aliases: List[str] = None,
        description: str = "",
        usage: str = "",
        examples: List[str] = None,
        permission: PermissionGroup = PermissionGroup.USER.value,
        timeout: float = None,
    ) -> Command:
        # TODO 提示已经注册的别名
        # TODO 限定参数
        if not hasattr(self, "_registered_commands"):
            self._registered_commands: List[Command] = []
        if name in self._registered_commands:
            raise ValueError(f"命令 {name} 已经存在")

        warpped_handler = self._create_command_handler(handler)
        func = self._register_func(
            name,
            warpped_handler,
            AliasFilter(name, aliases).check,
            description=description,
            usage=usage,
            examples=examples,
            permission=PermissionGroup.USER.value,
            timeout=timeout,
        )
        command = Command(
            {
                "name": name,
                "aliases": aliases,
                "permission": permission,
                "description": description,
                "usage": usage,
                "examples": examples,
                "func": func,
            }
        )
        self._registered_commands.append(command)
        return command

    def register_user_command(
        self,
        name: str,
        handler: Callable[[BaseMessageEvent, List[str]], Any],
        aliases: List[str] = None,
        description: str = "",
        usage: str = "",
        examples: List[str] = None,
        timeout: float = None,
    ) -> Command:
        return self.register_command(
            name,
            handler,
            aliases,
            description,
            usage,
            examples,
            PermissionGroup.USER.value,
            timeout,
        )

    def register_admin_command(
        self,
        name: str,
        handler: Callable[[List[str], BaseMessageEvent], Any],
        aliases: List[str] = None,
        description: str = "",
        usage: str = "",
        examples: List[str] = None,
        timeout: float = None,
    ) -> Command:
        return self.register_command(
            name,
            handler,
            aliases,
            description,
            usage,
            examples,
            PermissionGroup.ADMIN.value,
            timeout,
        )



from typing import Literal, Union, List
from .utils import BaseAPI, APIReturnStatus
from ncatbot.utils import run_coroutine
from ncatbot.core.event.message_segment.message_segment import convert_uploadable_object
from dataclasses import dataclass


@dataclass
class LoginInfo:
    nickname: str
    user_id: str


class CustomFaceList:
    urls: List[str]

    def __init__(self, data: List[dict]):
        self.urls = [item["url"] for item in data]


class AccountAPI(BaseAPI):
    # ---------------------
    # region 账号相关
    # ---------------------
    async def set_qq_profile(
        self, nickname: str, personal_note: str, sex: Literal["未知", "男", "女"]
    ) -> None:
        sex_id = str({"未知": 0, "男": 1, "女": 2}[sex])
        result = await self.async_callback(
            "/set_qq_profile",
            {"nickname": nickname, "personal_note": personal_note, "sex": sex_id},
        )
        APIReturnStatus.raise_if_failed(result)

    async def set_online_status(
        self, status: int, ext_status: int, battary_status: int
    ) -> None:
        result = await self.async_callback(
            "/set_online_status",
            {
                "status": status,
                "ext_status": ext_status,
                "battary_status": battary_status,
            },
        )
        APIReturnStatus.raise_if_failed(result)

    async def set_avatar(self, file: str) -> None:
        result = await self.async_callback(
            "/set_avatar", {"file": convert_uploadable_object(file)}
        )
        APIReturnStatus.raise_if_failed(result)

    async def set_self_longnick(self, longNick: str) -> None:
        result = await self.async_callback("/set_self_longnick", {"longNick": longNick})
        APIReturnStatus.raise_if_failed(result)

    async def get_login_info(self) -> LoginInfo:
        result = await self.async_callback("/get_login_info")
        status = APIReturnStatus(result)

        return LoginInfo(**status.data)

    async def get_status(self) -> dict:
        # TODO: 返回值(不紧急)
        result = await self.async_callback("/get_status")
        status = APIReturnStatus(result)

        return status.data

    # ---------------------
    # region 好友
    # ---------------------

    async def get_friends_with_cat(self) -> List[dict]:
        # TODO: 返回值(不紧急)
        result = await self.async_callback("/get_friends_with_cat")
        status = APIReturnStatus(result)

        return status.data

    async def send_like(self, user_id: Union[str, int], times: int = 1) -> None:
        result = await self.async_callback(
            "/send_like", {"user_id": user_id, "times": times}
        )
        APIReturnStatus.raise_if_failed(result)

    async def set_friend_add_request(
        self, flag: str, approve: bool, remark: str = None
    ) -> None:
        """设置通过好友请求

        Args:
            flag (str): 请求 flag
            approve (bool): 是否同意
            remark (str, optional): 通过后好友备注. Defaults to None.
        """
        result = await self.async_callback(
            "/set_friend_add_request",
            {"flag": flag, "approve": approve, "remark": remark},
        )
        APIReturnStatus.raise_if_failed(result)

    async def get_friend_list(self) -> List[dict]:
        # TODO: 返回值(不紧急)
        result = await self.async_callback("/get_friend_list")
        status = APIReturnStatus(result)

        return status.data

    async def delete_friend(
        self, user_id: Union[str, int], block: bool = True, both: bool = True
    ) -> None:
        """删除好友
        Args:
            user_id (Union[str, int]): 目标用户 QQ 号
            block (bool, optional): 是否拉黑. Defaults to True.
            both (bool, optional): 是否双向删除. Defaults to True.
        """
        result = await self.async_callback(
            "/delete_friend", {"user_id": user_id, "block": block, "both": both}
        )
        APIReturnStatus.raise_if_failed(result)

    async def set_friend_remark(self, user_id: Union[str, int], remark: str) -> None:
        result = await self.async_callback(
            "/set_friend_remark", {"user_id": user_id, "remark": remark}
        )
        APIReturnStatus.raise_if_failed(result)

    # ---------------------
    # region 消息
    # ---------------------

    async def mark_group_msg_as_read(self, group_id: Union[str, int]) -> None:
        result = await self.async_callback(
            "/mark_group_msg_as_read", {"group_id": group_id}
        )
        APIReturnStatus.raise_if_failed(result)

    async def mark_private_msg_as_read(self, user_id: Union[str, int]) -> None:
        result = await self.async_callback(
            "/mark_private_msg_as_read", {"user_id": user_id}
        )
        APIReturnStatus.raise_if_failed(result)

    async def create_collection(self, rawData: str, brief: str) -> None:
        result = await self.async_callback(
            "/create_collection", {"rawData": rawData, "brief": brief}
        )
        APIReturnStatus.raise_if_failed(result)

    async def get_recent_contact(self) -> List[dict]:
        # TODO: 返回值(不紧急)
        result = await self.async_callback("/get_recent_contact")
        status = APIReturnStatus(result)

        return status.data

    async def _mark_all_as_read(self) -> None:
        result = await self.async_callback("/_mark_all_as_read")
        APIReturnStatus.raise_if_failed(result)

    # ---------------------
    # region 群
    # ---------------------

    async def AskShareGroup(self, group_id: Union[str, int]) -> None:
        result = await self.async_callback("/AskShareGroup", {"group_id": group_id})
        APIReturnStatus.raise_if_failed(result)

    # ---------------------
    # region 其它
    # ---------------------
    async def get_stranger_info(self, user_id: Union[str, int]) -> dict:
        """获取陌生人信息，接口暂不完善，有需求提 PR
        : 解析参考 https://napcat.apifox.cn/226656970e0
        """
        # TODO: 返回值(不紧急)
        result = await self.async_callback("/get_stranger_info", {"user_id": user_id})
        status = APIReturnStatus(result)
        return status.data

    async def fetch_custom_face(self, count: int = 48) -> CustomFaceList:
        result = await self.async_callback("/fetch_custom_face", {"count": count})
        status = APIReturnStatus(result)
        return CustomFaceList(status.data)

    async def nc_get_user_status(self, user_id: Union[str, int]) -> dict:
        result = await self.async_callback("/nc_get_user_status", {"user_id": user_id})
        status = APIReturnStatus(result)

        return status.data

    # ---------------------
    # region 同步版本接口
    # ---------------------

    def set_qq_profile_sync(
        self, nickname: str, personal_note: str, sex: Literal["未知", "男", "女"]
    ) -> None:
        return run_coroutine(self.set_qq_profile, nickname, personal_note, sex)

    def set_online_status_sync(
        self, status: int, ext_status: int, battary_status: int
    ) -> None:
        return run_coroutine(self.set_online_status, status, ext_status, battary_status)

    def set_avatar_sync(self, file: str) -> None:
        return run_coroutine(self.set_avatar, file)

    def set_self_longnick_sync(self, longNick: str) -> None:
        return run_coroutine(self.set_self_longnick, longNick)

    def get_login_info_sync(self) -> LoginInfo:
        return run_coroutine(self.get_login_info)

    def get_status_sync(self) -> dict:
        return run_coroutine(self.get_status)

    def get_friends_with_cat_sync(self) -> List[dict]:
        return run_coroutine(self.get_friends_with_cat)

    def send_like_sync(self, user_id: Union[str, int], times: int = 1) -> None:
        return run_coroutine(self.send_like, user_id, times)

    def set_friend_add_request_sync(
        self, flag: str, approve: bool, remark: str = None
    ) -> None:
        return run_coroutine(self.set_friend_add_request, flag, approve, remark)

    def get_friend_list_sync(self) -> List[dict]:
        return run_coroutine(self.get_friend_list)

    def delete_friend_sync(
        self, user_id: Union[str, int], block: bool = True, both: bool = True
    ) -> None:
        return run_coroutine(self.delete_friend, user_id, block, both)

    def set_friend_remark_sync(self, user_id: Union[str, int], remark: str) -> None:
        return run_coroutine(self.set_friend_remark, user_id, remark)

    def mark_group_msg_as_read_sync(self, group_id: Union[str, int]) -> None:
        return run_coroutine(self.mark_group_msg_as_read, group_id)

    def mark_private_msg_as_read_sync(self, user_id: Union[str, int]) -> None:
        return run_coroutine(self.mark_private_msg_as_read, user_id)

    def create_collection_sync(self, rawData: str, brief: str) -> None:
        return run_coroutine(self.create_collection, rawData, brief)

    def get_recent_contact_sync(self) -> List[dict]:
        return run_coroutine(self.get_recent_contact)

    def _mark_all_as_read_sync(self) -> None:
        return run_coroutine(self._mark_all_as_read)

    def AskShareGroup_sync(self, group_id: Union[str, int]) -> None:
        return run_coroutine(self.AskShareGroup, group_id)

    def get_stranger_info_sync(self, user_id: Union[str, int]) -> dict:
        return run_coroutine(self.get_stranger_info, user_id)

    def fetch_custom_face_sync(self, count: int = 48) -> CustomFaceList:
        return run_coroutine(self.fetch_custom_face, count)

    def nc_get_user_status_sync(self, user_id: Union[str, int]) -> dict:
        return run_coroutine(self.nc_get_user_status, user_id)



"""命令解析器（Resolver）

职责：
- 从注册器构建命令索引
- 严格冲突检测（默认禁止前缀重叠）
- 基于 token 的命令匹配（首段文本 tokens）
"""

from __future__ import annotations

from dataclasses import dataclass
from typing import Dict, List, Optional, Tuple

from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system.utils.specs import (
    CommandSpec,
)
from ncatbot.utils import get_log
from ..command_system.lexer.tokenizer import Token, TokenType

LOG = get_log(__name__)


@dataclass
class CommandEntry:
    path_words: Tuple[str, ...]
    command: CommandSpec


class CommandResolver:
    def __init__(
        self, *, case_sensitive: bool, prefixes: List[str], allow_hierarchical: bool
    ) -> None:
        self.case_sensitive = case_sensitive
        self.allow_hierarchical = allow_hierarchical
        self.prefixes_tuple = tuple(prefix for prefix in prefixes)
        # 哈希索引：规范化路径字符串 → CommandEntry
        self._index: Dict[Tuple[str, ...], CommandEntry] = {}

    def clear(self):
        """清除索引"""
        self._index.clear()

    def _normalize(self, s: str) -> str:
        return s if self.case_sensitive else s.lower()

    def _path_to_norm(self, path: Tuple[str, ...]) -> Tuple[str, ...]:
        return tuple(self._normalize(p) for p in path)

    def build_index(
        self,
        commands: Dict[Tuple[str, ...], CommandSpec],
        aliases: Dict[Tuple[str, ...], CommandSpec],
    ) -> None:
        """构建索引并进行严格冲突检测。"""
        entries: Dict[Tuple[str, ...], CommandEntry] = {}

        def add_entry(path: Tuple[str, ...], command: CommandSpec):
            norm_path = self._path_to_norm(path)
            if norm_path in entries:
                raise ValueError(f"命令重复: {' '.join(norm_path)}")
            entries[norm_path] = CommandEntry(path_words=norm_path, command=command)

        for path, command in commands.items():
            add_entry(path, command)
        for path, command in aliases.items():
            add_entry(path, command)

        # 严格：不允许任意两条路径存在前缀关系
        paths = list(entries.keys())
        for i in range(len(paths)):
            for j in range(i + 1, len(paths)):
                a, b = paths[i], paths[j]
                min_len = min(len(a), len(b))
                if a[:min_len] == b[:min_len] and a != b:
                    if not self.allow_hierarchical:
                        raise ValueError(
                            f"命令前缀冲突: {' '.join(a)} vs {' '.join(b)}"
                        )

        self._index = entries
        LOG.debug(f"Resolver 索引构建完成: {len(self._index)}")

    def resolve_from_tokens(
        self, tokens: List[Token]
    ) -> Optional[Tuple[str, CommandEntry]]:
        """从首段 tokens 解析命令。

        策略：仅接收 TokenType.WORD/QUOTED_STRING 序列作为命令词；
        遇到第一个非上述类型 token 即停止命令词聚合。
        在严格模式下，应当保证不会出现“多匹配”。
        """
        words: List[str] = []
        for t in tokens:
            if t.type in (TokenType.WORD, TokenType.QUOTED_STRING):
                words.append(t.value)
            else:
                break

        if not words:
            return None, None
        # 尝试从最长到最短匹配（兼容 allow_hierarchical=true 的场景）
        for k in range(len(words), 0, -1):
            candidate = tuple(self._normalize(w) for w in words[:k])
            if candidate in self._index:
                return "", self._index[candidate]

        if words[0].startswith(self.prefixes_tuple):
            prefix = words[0][0]
            words[0] = words[0][1:]
            for k in range(len(words), 0, -1):
                candidate = tuple(self._normalize(w) for w in words[:k])
                if candidate in self._index:
                    return prefix, self._index[candidate]
        return None, None



# -------------------------
# @Author       : Fish-LP fish.zh@outlook.com
# @Date         : 2025-08-05 21:53:12
# @LastEditors  : Fish-LP fish.zh@outlook.com
# @LastEditTime : 2025-08-05 21:53:23
# @Description  : 喵喵喵, 我还没想好怎么介绍文件喵
# @Copyright (c) 2025 by Fish-LP, Fcatbot使用许可协议
# -------------------------

import functools
import warnings


class DeprecatedWarningState:
    def __init__(self):
        self.warned = set()


def deprecated(replacement=None):
    """
    装饰器，用于标记函数或方法为已弃用。
    Args:
        replacement: 可选参数，指定替代函数的名称。
    """
    state = DeprecatedWarningState()

    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            if func not in state.warned:
                msg = f"{func.__name__} is deprecated"
                if replacement:
                    msg += f", use {replacement} instead"
                warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
                state.warned.add(func)
            return func(*args, **kwargs)

        return wrapper

    return decorator



import importlib.metadata as _meta
import subprocess
import sys
import logging
from packaging.requirements import Requirement
from packaging.version import Version

LOG = logging.getLogger("PackageHelper")
# TODO 不兼容检测


class PackageHelper:
    """检查并确保某个 PEP-508 需求字符串被满足，否则自动安装/升级"""

    @staticmethod
    def ensure(req_str: str) -> None:
        req = Requirement(req_str)
        dist_name = req.name

        try:
            installed_ver = _meta.version(dist_name)
        except _meta.PackageNotFoundError:
            LOG.info("缺失包 %s，开始安装", dist_name)
            try:
                PackageHelper._pip_install(req_str)
            except subprocess.CalledProcessError:
                LOG.error("安装失败: %s", req_str)
                raise
            return

        # 已安装 → 检查版本规格
        if not req.specifier.contains(Version(installed_ver)):
            LOG.warning("版本不符(现有 %s, 需求 %s)，尝试升级", installed_ver, req_str)
            PackageHelper._pip_install(req_str)
        else:
            LOG.info("依赖已满足: %s", req_str)

    # ---------- 内部 ----------
    @staticmethod
    def _pip_install(spec: str) -> None:
        cmd = [sys.executable, "-m", "pip", "install", spec]
        LOG.debug("执行: %s", " ".join(cmd))
        subprocess.check_call(cmd)



"""命令规格数据类定义

提供类型安全的数据结构，替代字典存储选项、选项组和参数配置。
"""

from dataclasses import dataclass
from typing import Callable, Any, List, Optional, Dict, Type
import inspect


class FuncSpec:
    def __init__(self, func: Callable):
        self.func = func
        self.aliases = getattr(func, "__aliases__", [])

        # 生成 metadata 以便代码更易于理解
        self.func_name = func.__name__
        self.func_module = func.__module__
        self.func_qualname = func.__qualname__
        self.signature = inspect.signature(func)
        self.param_list = list(self.signature.parameters.values())
        self.param_names = [param.name for param in self.param_list]
        self.param_annotations = [param.annotation for param in self.param_list]


@dataclass
class OptionSpec:
    """选项规格数据类

    替代原有的选项字典存储。
    """

    short_name: Optional[str] = None
    long_name: Optional[str] = None
    description: str = ""

    @property
    def name(self) -> str:
        return self.long_name if self.long_name else self.short_name


@dataclass
class OptionGroupSpec:
    """选项组规格数据类

    替代原有的选项组字典存储。
    """

    choices: List[str]
    name: str
    default: str
    description: str = ""


@dataclass
class ParameterSpec:
    """参数规格数据类

    替代原有的参数字典存储。
    """

    name: str
    default: Any = None
    required: Optional[bool] = False
    description: str = ""
    choices: Optional[List[Any]] = None
    validator: Optional[Callable] = None
    examples: List[str] = None
    type_hints: Dict[Type, str] = None
    type_examples: Dict[Type, List[str]] = None
    is_named: bool = True
    is_positional: bool = False
    index: int = -1  # 用于确认参数位置（args_types 中），由分析器设置

    def __post_init__(self):
        if self.examples is None:
            self.examples = []
        if self.type_hints is None:
            self.type_hints = {}
        if self.type_examples is None:
            self.type_examples = {}


class CommandSpec:
    # 彻底分析函数后得到的命令标识器，用于指导参数传递
    def __init__(
        self,
        options: List[OptionSpec],
        option_groups: List[OptionGroupSpec],
        params: List[ParameterSpec],
        args_types: List[type],
        func: Callable,
    ):
        self.options: List[OptionSpec] = options
        self.option_groups: List[OptionGroupSpec] = option_groups
        self.params: List[ParameterSpec] = params
        self.args_types: List[type] = args_types
        self.func: Callable = func

        # 必须外部重新设置的属性
        self.aliases = []  # 默认为空列表而不是None
        self.description = None
        self.name = None
        self.prefixes = []

    def get_kw_binding(self, option: str) -> dict:
        for value in self.options:
            if value.name == option or value.short_name == option:
                return {value.name: True}
        for value in self.option_groups:
            if option in value.choices:
                return {value.name: option}
        return None

    def get_param_binding(self, param: str, value: Any) -> dict:
        for param_spec in self.params:
            if param_spec.name == param:
                target_type = self.args_types[param_spec.index]
                if target_type is bool:
                    return {param_spec.name: value.lower() not in ["false", "0"]}
                else:
                    return {param_spec.name: target_type(value)}
        return None



"""命令子系统

现代化的命令行参数解析系统，支持：
- 短选项：-v, -xvf
- 长选项：--verbose, --help
- 参数赋值：-p=1234, --para=value
- 转义字符串："有 空格 的 字符串"

使用示例：
    from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system import StringTokenizer, AdvancedCommandParser

    tokenizer = StringTokenizer('deploy app --env=prod -v "config file"')
    tokens = tokenizer.tokenize()
    parser = AdvancedCommandParser()
    result = parser.parse(tokens)
"""

from .analyzer import FuncAnalyser, get_subclass_recursive
from .registry import command_registry
from .lexer import (
    StringTokenizer,
    Token,
    TokenType,
    QuoteState,
    QuoteMismatchError,
    InvalidEscapeSequenceError,
    Element,
    ParsedCommand,
    AdvancedCommandParser,
    NonTextToken,
    MessageTokenizer,
    parse_message_command,
)

__all__ = [
    # 分析器
    "FuncAnalyser",
    "get_subclass_recursive",
    # 命令相关
    "CommandGroup",
    "CommandRouter",
    # 词法分析器
    "StringTokenizer",
    "Token",
    "TokenType",
    "QuoteState",
    "QuoteMismatchError",
    "InvalidEscapeSequenceError",
    # 命令解析器
    "Element",
    "ParsedCommand",
    "AdvancedCommandParser",
    "NonTextToken",
    # 消息级别解析器
    "MessageTokenizer",
    "parse_message_command",
    # 注册器实例
    "command_registry",
]



# -------------------------
# @Author       : Fish-LP fish.zh@outlook.com
# @Date         : 2025-06-12 18:58:49
# @LastEditors  : Fish-LP fish.zh@outlook.com
# @LastEditTime : 2025-08-04 14:55:45
# @Description  : 喵喵喵, 我还没想好怎么介绍文件喵
# @Copyright (c) 2025 by Fish-LP, Fcatbot使用许可协议
# -------------------------
from dataclasses import dataclass, field


@dataclass
class PluginSystemConfig:
    """
    插件系统配置类，用于管理插件系统的全局配置。
    """

    auto_install_pip_pack: bool = field(default=True)
    """是否自动安装依赖的 pip 包"""
    plugins_dir: str = field(default="./plugins")
    """插件目录路径"""
    plugins_data_dir: str = field(default="./data")
    plugins_config_dir: str = field(default="./config")
    rbac_path: str = field(default="./data/rbac.json")
    """RBAC 配置文件路径"""


config = PluginSystemConfig()



from .event_data import BaseEventData
from typing import Literal, Optional
from dataclasses import dataclass


class File:
    id: str = None
    name: str = None
    size: str = None
    busid: str = None


@dataclass
class NoticeEvent(BaseEventData):
    # 保留细化能力
    post_type: Literal["notice"] = None
    time: int = None
    self_id: str = None  # 和 OneBot11 标准不一致, 这里采取 str
    notice_type: Literal[
        "group_upload",
        "group_admin",
        "group_decrease",
        "group_increase",
        "friend_add",
        "group_recall",
        "group_ban",
        "notify",
    ] = None
    sub_type: Literal[
        "set",
        "unset",
        "leave",
        "kick",
        "kick_me",
        "approve",
        "invite",
        "ban",
        "lift_ban",
        "poke",
        "lucky_king",
        "honor",
    ] = None
    group_id: int = None
    user_id: int = None
    file: Optional[File] = None  # group_upload
    operator_id: Optional[str] = (
        None  # group_decrease, group_increase, group_ban, group_recall
    )
    raw_info: Optional[dict] = None  # notify
    duration: Optional[int] = None  # group_ban
    card_old: Optional[str] = None  # group_increase
    card_new: Optional[str] = None  # group_increase
    message_id: Optional[str] = None  # group_recall, friend_recall
    target_id: Optional[str] = None  # notify.poke, notify.lucky_king
    honor_type: Optional[Literal["talkative", "performer", "emotion"]] = (
        None  # notify.honor
    )

    def get_core_properties_str(self):
        core_properties = []
        for k, v in self.__dict__.items():
            if v is not None:
                core_properties.append(f"{k}={v}")
        core_properties.remove("post_type=notice")
        core_properties.remove(f"time={self.time}")
        core_properties.remove(f"self_id={self.self_id}")
        return core_properties

    def __repr__(self):
        return f"{self.__class__.__name__}({', '.join(self.get_core_properties_str())})"

    def __str__(self):
        return self.__repr__()



from typing import Literal
from ncatbot.core.event.event_data import BaseEventData
from ncatbot.utils import status, get_log

LOG = get_log("ncatbot.core.event.request")


class RequestEvent(BaseEventData):
    post_type: Literal["request"] = None
    request_type: Literal["friend", "group"] = None
    comment: str = None  # 验证信息
    flag: str = None  # 验证 flag

    def is_friend_request(self) -> bool:
        return self.request_type == "friend"

    def is_group_request(self) -> bool:
        return self.request_type == "group"

    async def approve(
        self, approve: bool = True, remark: str = None, reason: str = None
    ):
        """通过或者拒绝验证
        Args:
            approve (bool, optional): 是否通过. Defaults to True.
            remark (str, optional): 如果是好友请求, 此选项表示好友备注, 否则无效. Defaults to None.
            reason (str, optional): 如果是加群请求, 此选项表示拒绝理由, 否则无效. Defaults to None.
        """
        if self.request_type == "friend":
            if reason is not None:
                LOG.warning("好友请求不支持拒绝理由")
            return await status.global_api.set_friend_add_request(
                self.flag, approve, remark
            )
        elif self.request_type == "group":
            if remark is not None:
                LOG.warning("加群请求不支持备注")
            return await status.global_api.set_group_add_request(
                self.flag, approve, reason
            )

    def approve_sync(
        self, approve: bool = True, remark: str = None, reason: str = None
    ):
        if self.request_type == "friend":
            if reason is not None:
                LOG.warning("好友请求不支持拒绝理由")
            return status.global_api.set_friend_add_request_sync(
                self.flag, approve, remark
            )
        elif self.request_type == "group":
            if remark is not None:
                LOG.warning("加群请求不支持备注")
            return status.global_api.set_group_add_request_sync(
                self.flag, approve, reason
            )

    def __init__(self, data):
        super().__init__(data)
        self.request_type = data.get("request_type")
        self.comment = data.get("comment")
        self.flag = data.get("flag")



"""消息级别分词器

处理 MessageArray，将其转换为 Token 序列，支持文本和非文本元素混合处理。
"""

from typing import List
from .tokenizer import (
    StringTokenizer,
    Token,
    TokenType,
    NonTextToken,
    AdvancedCommandParser,
    ParsedCommand,
)


class MessageTokenizer:
    """消息级别分词器

    将 MessageArray 转换为 Token 序列，支持：
    - PlainText 及其子类：使用 StringTokenizer 分析
    - 其他 MessageSegment：转换为 NonTextToken
    """

    def __init__(self):
        self.command_parser = AdvancedCommandParser()

    def tokenize(self, message_array) -> List[Token]:
        """将 MessageArray 转换为 Token 序列

        Args:
            message_array: MessageArray 对象

        Returns:
            Token 序列，包含文本 Token 和 NonTextToken
        """
        tokens = []

        for segment in message_array.messages:
            if self._is_text_segment(segment):
                # 文本段：使用 StringTokenizer 分析
                text_tokenizer = StringTokenizer(segment.text)
                text_tokens = text_tokenizer.tokenize()

                # 过滤 EOF，添加到结果中
                for token in text_tokens:
                    if token.type != TokenType.EOF:
                        # 调整 token 位置为全局位置
                        token.position = len(tokens)
                        tokens.append(token)
            else:
                # 非文本段：创建 NonTextToken
                non_text_token = NonTextToken(segment, len(tokens))
                tokens.append(non_text_token)

        # 添加最终的 EOF
        tokens.append(Token(TokenType.EOF, "", len(tokens)))
        return tokens

    def parse_message(self, message_array) -> ParsedCommand:
        """直接从 MessageArray 解析到 ParsedCommand

        Args:
            message_array: MessageArray 对象

        Returns:
            ParsedCommand 解析结果
        """
        # 1. 分词
        tokens = self.tokenize(message_array)

        # 2. 解析
        result = self.command_parser.parse(tokens)

        return result

    def _is_text_segment(self, segment) -> bool:
        """判断是否为文本段

        Args:
            segment: MessageSegment 对象

        Returns:
            是否为文本段
        """
        # 检查是否有 text 属性且 msg_seg_type 为 text 或其子类
        return (
            hasattr(segment, "text")
            and hasattr(segment, "msg_seg_type")
            and segment.msg_seg_type in ["text", "plain_text"]
        )


# 便捷函数
def parse_message_command(message_array) -> ParsedCommand:
    """便捷函数：直接解析 MessageArray

    Args:
        message_array: MessageArray 对象

    Returns:
        ParsedCommand 解析结果
    """
    tokenizer = MessageTokenizer()
    return tokenizer.parse_message(message_array)



import asyncio
import inspect
import traceback
import threading
from typing import (
    Callable,
    Optional,
    Type,
    Literal,
    Union,
    TypedDict,
    List,
    TypeVar,
    Dict,
    TYPE_CHECKING,
)
from typing_extensions import Unpack

if TYPE_CHECKING:
    from ncatbot.plugin_system import BasePlugin

from .adapter import lanuch_napcat_service, Adapter
from .api import BotAPI
from .event import (
    MessageSegment,
    BaseEventData,
    PrivateMessageEvent,
    GroupMessageEvent,
    NoticeEvent,
    RequestEvent,
    MetaEvent,
)
from ..utils import get_log, run_coroutine, ThreadPool
from ..utils import status, ncatbot_config
from ..utils import NcatBotError, NcatBotConnectionError
from ..utils import (
    OFFICIAL_PRIVATE_MESSAGE_EVENT,
    OFFICIAL_GROUP_MESSAGE_EVENT,
    OFFICIAL_NOTICE_EVENT,
    OFFICIAL_REQUEST_EVENT,
    OFFICIAL_STARTUP_EVENT,
    OFFICIAL_SHUTDOWN_EVENT,
    OFFICIAL_HEARTBEAT_EVENT,
)

T = TypeVar("T")
LOG = get_log("Client")
EVENTS = (
    OFFICIAL_PRIVATE_MESSAGE_EVENT,
    OFFICIAL_GROUP_MESSAGE_EVENT,
    OFFICIAL_NOTICE_EVENT,
    OFFICIAL_REQUEST_EVENT,
    OFFICIAL_STARTUP_EVENT,
    OFFICIAL_SHUTDOWN_EVENT,
    OFFICIAL_HEARTBEAT_EVENT,
)


class StartArgs(TypedDict, total=False):
    bt_uin: Optional[int]
    root: Optional[str]
    ws_uri: Optional[str]
    webui_uri: Optional[str]
    ws_token: Optional[str]
    webui_token: Optional[str]
    ws_listen_ip: Optional[str]
    remote_mode: Optional[bool]
    enable_webui_interaction: Optional[bool]
    debug: Optional[bool]
    # 以后再加参数直接在这里补一行即可，无需改函数签名


class BotClient:
    _initialized = False  # 兼容旧版本检查

    def __init__(self, only_private: bool = False):
        if self._initialized:
            raise NcatBotError("BotClient 实例只能创建一次")
        self._initialized = True
        self.adapter = Adapter()
        self.event_handlers: Dict[str, list] = {}
        self.thread_pool = ThreadPool(max_workers=1, max_per_func=1)
        self.api = BotAPI(self.adapter.send)
        self.crash_flag = False
        status.global_api = self.api
        for event_name in EVENTS:
            self.create_official_event_handler_group(event_name)

        self.register_builtin_handler(only_private=only_private)

    def register_builtin_handler(self, only_private: bool = False):
        # 注册插件系统事件处理器
        def make_async_handler(event_name):
            async def wrapper(event: BaseEventData):
                LOG.debug(f"已发布 {event_name} 事件")
                from ncatbot.plugin_system.event import NcatBotEvent

                await self.event_bus.publish(NcatBotEvent(event_name, event))

            return wrapper

        if only_private:
            self.add_private_message_handler(
                make_async_handler(OFFICIAL_PRIVATE_MESSAGE_EVENT)
            )
        else:
            self.add_startup_handler(lambda x: LOG.info(f"Bot {x.self_id} 启动成功"))
            self.add_startup_handler(make_async_handler(OFFICIAL_STARTUP_EVENT))
            self.add_private_message_handler(
                make_async_handler(OFFICIAL_PRIVATE_MESSAGE_EVENT)
            )
            self.add_group_message_handler(
                make_async_handler(OFFICIAL_GROUP_MESSAGE_EVENT)
            )
            self.add_notice_handler(make_async_handler(OFFICIAL_NOTICE_EVENT))
            self.add_request_handler(make_async_handler(OFFICIAL_REQUEST_EVENT))
            self.add_shutdown_handler(make_async_handler(OFFICIAL_SHUTDOWN_EVENT))
            self.add_heartbeat_handler(make_async_handler(OFFICIAL_HEARTBEAT_EVENT))

    def create_official_event_handler_group(self, event_name):
        async def event_callback(event: BaseEventData):
            # 处理回调, 不能阻塞
            for handler in self.event_handlers[event_name]:
                self.thread_pool.submit(handler, event)

        self.adapter.event_callback[event_name] = event_callback
        self.event_handlers[event_name] = []

    def add_handler(self, event_name, handler):
        self.event_handlers[event_name].append(handler)

    # 计划为 filter 提供全面支持, 会直接从 MessageArray 中过滤
    def add_group_message_handler(
        self, handler: Callable[[GroupMessageEvent], None], filter=None
    ):
        async def wrapper(event: GroupMessageEvent):
            new_messages = event.message.filter(filter)
            if len(new_messages) == 0:
                return
            if inspect.iscoroutinefunction(handler):
                await handler(event)
            else:
                handler(event)

        self.add_handler(OFFICIAL_GROUP_MESSAGE_EVENT, wrapper)

    def add_private_message_handler(
        self, handler: Callable[[PrivateMessageEvent], None], filter=None
    ):
        async def wrapper(event: PrivateMessageEvent):
            new_messages = event.message.filter(filter)
            if len(new_messages) == 0:
                return
            if inspect.iscoroutinefunction(handler):
                await handler(event)
            else:
                handler(event)

        self.add_handler(OFFICIAL_PRIVATE_MESSAGE_EVENT, wrapper)

    def add_notice_handler(self, handler: Callable[[NoticeEvent], None], filter=None):
        self.add_handler(OFFICIAL_NOTICE_EVENT, handler)

    def add_request_handler(
        self, handler: Callable[[RequestEvent], None], filter=Literal["group", "friend"]
    ):
        async def wrapper(event: RequestEvent):
            if filter != event.request_type:
                return
            if inspect.iscoroutinefunction(handler):
                await handler(event)
            else:
                handler(event)

        self.add_handler(OFFICIAL_REQUEST_EVENT, wrapper)

    def add_startup_handler(self, handler: Callable[[MetaEvent], None]):
        self.add_handler(OFFICIAL_STARTUP_EVENT, handler)

    def add_shutdown_handler(self, handler: Callable[[MetaEvent], None]):
        self.add_handler(OFFICIAL_SHUTDOWN_EVENT, handler)

    def add_heartbeat_handler(self, handler: Callable[[MetaEvent], None]):
        self.add_handler(OFFICIAL_HEARTBEAT_EVENT, handler)

    # 装饰器版本 ==========================================
    def on_group_message(
        self,
        filter: Union[Type[MessageSegment], None] = None,
    ):
        """装饰器注册群消息处理器"""
        if filter is not None and not issubclass(filter, MessageSegment):
            raise TypeError("filter 必须是 MessageSegment 的子类")

        def decorator(f: Callable[[GroupMessageEvent], None]):
            self.add_group_message_handler(f, filter)
            return f  # 其实没有必要

        return decorator

    def on_private_message(
        self,
        filter: Union[Type[MessageSegment], None] = None,
    ):
        """装饰器注册私聊消息处理器"""
        if filter is not None and not issubclass(filter, MessageSegment):
            raise TypeError("filter 必须是 MessageSegment 的子类")

        def decorator(f: Callable[[PrivateMessageEvent], None]):
            self.add_private_message_handler(f, filter)
            return f  # 其实没有必要

        return decorator

    def on_notice(self, filter=None):
        """装饰器注册通知事件处理器"""

        def decorator(f: Callable[[NoticeEvent], None]):
            self.add_notice_handler(f, filter)
            return f

        return decorator

    def on_request(self, filter=Literal["group", "friend"]):
        """装饰器注册请求事件处理器"""

        def decorator(f: Callable[[RequestEvent], None]):
            self.add_request_handler(f, filter)
            return f

        return decorator

    def on_startup(self):
        """装饰器注册启动事件处理器"""

        def decorator(f: Callable[[MetaEvent], None]):
            self.add_startup_handler(f)
            return f

        return decorator

    def on_shutdown(self):
        """装饰器注册关闭事件处理器"""

        def decorator(f: Callable[[MetaEvent], None]):
            self.add_shutdown_handler(f)
            return f

        return decorator

    def on_heartbeat(self):
        """装饰器注册心跳事件处理器"""

        def decorator(f: Callable[[MetaEvent], None]):
            self.add_heartbeat_handler(f)
            return f

        return decorator

    def bot_exit(self):
        status.exit = True
        asyncio.run(self.plugin_loader.unload_all())
        LOG.info("Bot 已经正常退出")

    def run_frontend(self, **kwargs: Unpack[StartArgs]):
        try:
            self.start(**kwargs)
        except KeyboardInterrupt:
            self.bot_exit()
        except Exception:
            raise

    def run_backend(self, **kwargs: Unpack[StartArgs]):
        def run_async_task():
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            try:
                self.start(**kwargs)
            except Exception as e:
                LOG.error(f"Bot 启动失败: {e}")
                LOG.info(traceback.format_exc())
            finally:
                loop.close()
                self.crash_flag = True
                self.release_callback(None)

        thread = threading.Thread(target=run_async_task)
        thread.daemon = True  # 设置为守护线程
        self.lock = threading.Lock()
        self.lock.acquire()
        self.release_callback = lambda x: self.lock.release()
        self.add_startup_handler(self.release_callback)
        thread.start()
        flag = self.lock.acquire(timeout=90)
        if self.crash_flag:
            raise NcatBotError("Bot 启动失败", log=False)
        if not flag:
            raise NcatBotError("Bot 启动超时", log=True)
        return self.api

    def start(self, **kwargs):
        # 配置参数
        legal_args = [
            "bt_uin",
            "root",
            "ws_uri",
            "webui_uri",
            "ws_token",
            "webui_token",
            "ws_listen_ip",
            "remote_mode",
            "enable_webui_interaction",
            "debug",
            "skip_plugin_load",
        ]
        for key, value in kwargs.items():
            if key not in legal_args:
                raise NcatBotError(f"非法参数: {key}")
            elif value is None:
                continue
            else:
                ncatbot_config.update_value(key, value)

        ncatbot_config.validate_config()

        # 加载插件
        from ncatbot.plugin_system import EventBus, PluginLoader

        self.event_bus = EventBus()
        self.plugin_loader = PluginLoader(self.event_bus, debug=ncatbot_config.debug)

        run_coroutine(self.plugin_loader.load_plugins)

        if getattr(self, "mock_mode", False):  # MockMixin 中提供
            self.mock_start()
        else:
            # 启动服务（仅在非 mock 模式下）
            lanuch_napcat_service()
            try:
                asyncio.run(self.adapter.connect_websocket())
            except NcatBotConnectionError:
                self.bot_exit()
                raise

    def get_registered_plugins(self) -> List["BasePlugin"]:
        return list(self.plugin_loader.plugins.values())

    def get_plugin(self, type: Type[T]) -> T:
        for plugin in self.get_registered_plugins():
            if isinstance(plugin, type):
                return plugin
        raise ValueError(f"插件 {type.__name__} 未找到")

    # 兼容 3xx 版本
    group_event = on_group_message
    private_event = on_private_message
    notice_event = on_notice
    request_event = on_request
    startup_event = on_startup
    shutdown_event = on_shutdown
    heartbeat_event = on_heartbeat
    add_group_event_handler = add_group_message_handler
    add_private_event_handler = add_private_message_handler
    add_notice_event_handler = add_notice_handler
    add_request_event_handler = add_request_handler
    add_startup_event_handler = add_startup_handler
    run_blocking = run_frontend
    run_non_blocking = run_backend
    run = run_frontend



import re
from pathlib import Path
from setuptools import setup, find_packages

# 基础配置
PACKAGE_NAME = "ncatbot"
AUTHOR = "木子"
EMAIL = "lyh_02@qq.com"
URL = "https://github.com/liyihao1110/ncatbot"
DESCRIPTION = "NcatBot，基于协议的QQ机器人Python SDK"
LICENSE = "MIT"


def get_version():
    """从__init__.py获取版本号"""
    init_file = Path(__file__).parent / "ncatbot" / "__init__.py"
    with open(init_file, encoding="utf-8") as f:
        version_match = re.search(
            r"^__version__\s*=\s*['\"]([^'\"]*)['\"]", f.read(), re.M
        )
    if version_match:
        return version_match.group(1)
    raise RuntimeError(f"无法在 {init_file} 中找到版本信息")


def get_requirements():
    """获取依赖列表"""
    req_file = Path(__file__).parent / "requirements.txt"
    with open(req_file, encoding="utf-8") as f:
        return [line.strip() for line in f if line.strip() and not line.startswith("#")]


def get_long_description():
    """获取长描述"""
    readme = Path(__file__).parent / "README.md"
    return readme.read_text(encoding="utf-8")


# 主配置
setup(
    # 基础元数据
    name=PACKAGE_NAME,
    version=get_version(),
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    description=DESCRIPTION,
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    license=LICENSE,
    # 包配置
    packages=find_packages(
        include=["ncatbot", "ncatbot.*"], exclude=["tests*", "docs*", "examples*"]
    ),
    package_dir={"": "."},  # 从根目录查找包
    include_package_data=True,
    zip_safe=False,
    # 依赖管理
    install_requires=get_requirements(),
    python_requires=">=3.8",
    # 入口点配置（确保路径正确）
    entry_points={
        "console_scripts": [
            "ncatbot=ncatbot.cli.main:main",
        ],
    },
    # 分类信息
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    # 项目URLs
    project_urls={
        "Source": URL,
        "Bug Reports": f"{URL}/issues",
        "Documentation": f"{URL}/wiki",
    },
)



"""帮助系统

自动生成命令帮助文档和使用说明。
"""

from typing import List, Optional
from ..utils.specs import ParameterSpec, OptionSpec, OptionGroupSpec
from .registry import CommandSpec, CommandGroup


class HelpGenerator:
    """帮助文档生成器"""

    def __init__(self):
        self.max_width = 80
        self.indent = "  "

    def generate_command_help(self, cmd_def: CommandSpec) -> str:
        """生成单个命令的帮助文档"""
        lines = []

        # 命令头部
        lines.append(f"📋 命令: {cmd_def.name}")
        if cmd_def.aliases:
            lines.append(f"📋 别名: {', '.join(cmd_def.aliases)}")

        # 描述
        if cmd_def.description:
            lines.append("")
            lines.append("📝 描述:")
            lines.append(f"{self.indent}{cmd_def.description}")

        # 用法
        usage = self._generate_usage(cmd_def)
        lines.append("")
        lines.append("💡 用法:")
        lines.append(f"{self.indent}{usage}")

        # 位置参数
        positional_params = cmd_def.get_positional_parameters()
        if positional_params:
            lines.append("")
            lines.append("📍 位置参数:")
            for param in positional_params:
                lines.extend(self._format_parameter(param))

        # 命名参数
        named_params = cmd_def.get_named_parameters()
        if named_params:
            lines.append("")
            lines.append("🏷️ 命名参数:")
            for param in named_params:
                lines.extend(self._format_named_parameter(param))

        # 选项
        if cmd_def.options:
            lines.append("")
            lines.append("⚙️ 选项:")
            for option in cmd_def.options:
                lines.extend(self._format_option(option))

        # 选项组
        if cmd_def.option_groups:
            lines.append("")
            lines.append("📦 选项组:")
            for group in cmd_def.option_groups:
                lines.extend(self._format_option_group(group, cmd_def.options))

        # 示例
        examples = self._generate_examples(cmd_def)
        if examples:
            lines.append("")
            lines.append("🌰 示例:")
            for example in examples:
                lines.append(f"{self.indent}{example}")

        return "\n".join(lines)

    def generate_group_help(self, group: CommandGroup) -> str:
        """生成命令组的帮助文档"""
        lines = []

        # 组头部
        lines.append(f"📁 命令组: {group.name}")
        if group.description:
            lines.append(f"📝 描述: {group.description}")

        # 子命令
        if group.commands:
            lines.append("")
            lines.append("🔧 可用命令:")
            for name, cmd_def in group.commands.items():
                if name == cmd_def.name:  # 避免显示别名
                    desc = cmd_def.description or "无描述"
                    lines.append(f"{self.indent}{name:<20} {desc}")

        # 子组
        if group.subgroups:
            lines.append("")
            lines.append("📁 子命令组:")
            for name, subgroup in group.subgroups.items():
                desc = subgroup.description or "无描述"
                lines.append(f"{self.indent}{name:<20} {desc}")

        return "\n".join(lines)

    def generate_command_list(self, commands: List[CommandSpec]) -> str:
        """生成命令列表"""
        lines = ["📋 所有可用命令:"]
        lines.append("")

        # 按名称排序
        sorted_commands = sorted(commands, key=lambda x: x.name)

        for cmd_def in sorted_commands:
            desc = cmd_def.description or "无描述"
            # 截断过长的描述
            if len(desc) > 50:
                desc = desc[:47] + "..."
            lines.append(f"{self.indent}{cmd_def.name:<20} {desc}")

        lines.append("")
        lines.append("💡 使用 /<命令名> --help 查看详细帮助")

        return "\n".join(lines)

    def _generate_usage(self, cmd_def: CommandSpec) -> str:
        """生成用法字符串"""
        parts = [f"/{cmd_def.name}"]

        # 位置参数
        for param in cmd_def.get_positional_parameters():
            if param.required:
                parts.append(f"<{param.name}>")
            else:
                parts.append(f"[{param.name}]")

        # 命名参数
        named_params = cmd_def.get_named_parameters()
        if named_params:
            parts.append("[命名参数...]")

        # 选项
        if cmd_def.options:
            parts.append("[选项...]")

        return " ".join(parts)

    def _format_parameter(self, param: ParameterSpec) -> List[str]:
        """格式化位置参数"""
        lines = []

        # 参数名和类型
        type_name = param.get_friendly_type_name()
        param_line = f"{self.indent}{param.name}"
        if not param.required:
            param_line += " (可选)"
        param_line += f" - {type_name}"
        lines.append(param_line)

        # 描述
        if param.description:
            lines.append(f"{self.indent}{self.indent}{param.description}")

        # 默认值
        if not param.required and param.default is not None:
            lines.append(f"{self.indent}{self.indent}默认值: {param.default}")

        # 选择值
        if param.choices:
            choices_str = ", ".join(str(c) for c in param.choices)
            lines.append(f"{self.indent}{self.indent}可选值: {choices_str}")

        # 多类型示例
        if param.is_multi_type():
            union_type = param.get_union_type()
            for type_obj in union_type.types:
                examples = param.get_examples_for_type(type_obj)
                if examples:
                    type_name = type_obj.__name__
                    examples_str = ", ".join(examples[:3])
                    lines.append(
                        f"{self.indent}{self.indent}{type_name}示例: {examples_str}"
                    )

        return lines

    def _format_named_parameter(self, param: ParameterSpec) -> List[str]:
        """格式化命名参数"""
        lines = []

        # 参数名和类型
        type_name = param.get_friendly_type_name()
        param_line = f"{self.indent}--{param.name}"
        if not param.required:
            param_line += " (可选)"
        param_line += f" - {type_name}"
        lines.append(param_line)

        # 描述和其他信息同位置参数
        lines.extend(self._format_parameter(param)[1:])  # 跳过第一行

        return lines

    def _format_option(self, option: OptionSpec) -> List[str]:
        """格式化选项"""
        lines = []

        # 选项名
        names = option.get_option_names()
        option_line = f"{self.indent}{', '.join(names)}"

        if option.needs_value():
            type_name = (
                option.value_type.__name__
                if hasattr(option.value_type, "__name__")
                else str(option.value_type)
            )
            option_line += f" <{type_name}>"

        lines.append(option_line)

        # 描述
        if option.description:
            lines.append(f"{self.indent}{self.indent}{option.description}")

        # 默认值
        if option.default_value is not None and not option.is_flag():
            lines.append(f"{self.indent}{self.indent}默认值: {option.default_value}")

        # 选择值
        if option.choices:
            choices_str = ", ".join(str(c) for c in option.choices)
            lines.append(f"{self.indent}{self.indent}可选值: {choices_str}")

        return lines

    def _format_option_group(
        self, group: OptionGroupSpec, all_options: List[OptionSpec]
    ) -> List[str]:
        """格式化选项组"""
        lines = []

        # 组名
        group_line = f"{self.indent}{group.name}"
        if group.is_required:
            group_line += " (必选)"
        lines.append(group_line)

        # 组描述
        if group.description:
            lines.append(f"{self.indent}{self.indent}{group.description}")

        # 组内选项 - 显示所有可选项
        for choice in group.choices:
            choice_line = f"{self.indent}--{choice}"
            if choice == group.default:
                choice_line += " (默认)"
            lines.append(choice_line)

        return lines

    def _generate_examples(self, cmd_def: CommandSpec) -> List[str]:
        """生成使用示例"""
        examples = []

        # 基本示例
        basic_example = f"/{cmd_def.name}"

        # 添加必需的位置参数
        required_pos = [p for p in cmd_def.get_positional_parameters() if p.required]
        for param in required_pos:
            examples_for_type = param.get_examples_for_type(param.get_type_list()[0])
            if examples_for_type:
                basic_example += f" {examples_for_type[0]}"
            else:
                basic_example += f" <{param.name}>"

        examples.append(basic_example)

        # 带选项的示例
        if cmd_def.options:
            with_options = basic_example
            flag_options = [opt for opt in cmd_def.options if opt.is_flag()]
            if flag_options:
                opt = flag_options[0]
                if opt.short_name:
                    with_options += f" {opt.short_name}"
                elif opt.long_name:
                    with_options += f" {opt.long_name}"

            value_options = [opt for opt in cmd_def.options if opt.needs_value()]
            if value_options:
                opt = value_options[0]
                option_name = opt.long_name or opt.short_name
                if opt.choices:
                    value = opt.choices[0]
                elif opt.default_value is not None:
                    value = opt.default_value
                else:
                    value = "value"
                with_options += f" {option_name}={value}"

            if with_options != basic_example:
                examples.append(with_options)

        return examples


def format_error_with_help(
    error_msg: str, cmd_def: Optional[CommandSpec] = None
) -> str:
    """格式化错误信息并附加帮助提示"""
    lines = [f"❌ {error_msg}"]

    if cmd_def:
        lines.append("")
        lines.append("💡 正确用法:")

        # 生成简化的用法提示
        help_gen = HelpGenerator()
        usage = help_gen._generate_usage(cmd_def)
        lines.append(f"   {usage}")

        lines.append("")
        lines.append(f"📖 详细帮助: /{cmd_def.name} --help")

    return "\n".join(lines)



from typing import Union
from .utils import BaseAPI, APIReturnStatus
from ncatbot.utils import run_coroutine


class PrivateAPI(BaseAPI):
    # ---------------------
    # region 文件
    # ---------------------
    async def upload_private_file(
        self, user_id: Union[str, int], file: str, name: str
    ) -> None:
        result = await self.async_callback(
            "/upload_private_file", {"user_id": user_id, "file": file, "name": name}
        )
        APIReturnStatus.raise_if_failed(result)

    async def get_private_file_url(self, file_id: str) -> str:
        result = await self.async_callback(
            "/get_private_file_url", {"file_id": file_id}
        )
        status = APIReturnStatus(result)
        return status.data.get("url")

    async def post_private_file(
        self,
        user_id: Union[str, int],
        image: str = None,
        record: str = None,
        video: str = None,
        file: str = None,
    ) -> str:
        count = sum(1 for arg in [image, record, video, file] if arg is not None)
        if count != 1:
            raise ValueError("只能上传一个文件")
        if image is not None:
            return await self.send_private_image(user_id, image)
        elif record is not None:
            return await self.send_private_record(user_id, record)
        elif video is not None:
            return await self.send_private_video(user_id, video)
        elif file is not None:
            return await self.send_private_file(user_id, file)
        else:
            raise ValueError("必须上传一个文件")

    # ---------------------
    # region 其它
    # ---------------------
    async def set_input_status(self, status: int) -> None:
        """设置输入状态

        Args:
            status (int): 状态码, 0 表示 "对方正在说话", 1 表示 "对方正在输入"
        """
        result = await self.async_callback("/set_input_status", {"status": status})
        APIReturnStatus.raise_if_failed(result)

    # ---------------------
    # region 同步版本接口
    # ---------------------

    def upload_private_file_sync(
        self, user_id: Union[str, int], file: str, name: str
    ) -> None:
        return run_coroutine(self.upload_private_file, user_id, file, name)

    def get_private_file_url_sync(self, file_id: str) -> str:
        return run_coroutine(self.get_private_file_url, file_id)

    def post_private_file_sync(
        self,
        user_id: Union[str, int],
        image: str = None,
        record: str = None,
        video: str = None,
        file: str = None,
    ) -> str:
        return run_coroutine(
            self.post_private_file, user_id, image, record, video, file
        )

    def set_input_status_sync(self, status: int) -> None:
        return run_coroutine(self.set_input_status, status)



from .client import BotClient
from .event import (
    GroupMessageEvent,
    PrivateMessageEvent,
    RequestEvent,
    NoticeEvent,
    MetaEvent,
    BaseMessageEvent,
)
from .event import MessageArray as MessageChain
from .event import BaseMessageEvent as BaseMessage
from .event import GroupMessageEvent as GroupMessage
from .event import PrivateMessageEvent as PrivateMessage
from .helper import ForwardConstructor
from .event import (
    Text,
    At,
    Image,
    Face,
    Reply,
    File,
    Record,
    Video,
    Node,
    Forward,
    MessageArray,
    MessageSegment,
)

__all__ = [
    "BaseMessage",
    "GroupMessage",
    "PrivateMessage",
    "BotClient",
    "GroupMessageEvent",
    "PrivateMessageEvent",
    "RequestEvent",
    "NoticeEvent",
    "MetaEvent",
    "ForwardConstructor",
    "MessageChain",
    "Text",
    "At",
    "Image",
    "Face",
    "Reply",
    "File",
    "Record",
    "Video",
    "Node",
    "Forward",
    "MessageArray",
    "MessageSegment",
    "BaseMessageEvent",
]



from typing import List
from ...utils import status, run_coroutine
from ..event import (
    MessageArray,
    Image,
    Text,
    Forward,
    Node,
    File,
    Video,
)


class ForwardConstructor:
    def __init__(
        self,
        user_id: str = "123456",
        nickname: str = "QQ用户",
        content: List[Node] = None,
    ):
        self.user_id = user_id
        self.nickname = nickname
        self.content = content if content else []

    def attach(self, content: MessageArray, user_id: str = None, nickname: str = None):
        self.content.append(
            Node(
                user_id=user_id if user_id else self.user_id,
                nickname=nickname if nickname else self.nickname,
                content=content,
            )
        )

    def attach_text(self, text: str, user_id: str = None, nickname: str = None):
        self.attach(MessageArray(Text(text)), user_id, nickname)

    def attach_image(self, image: str, user_id: str = None, nickname: str = None):
        self.attach(MessageArray(Image(image)), user_id, nickname)

    def attach_file(self, file: str, user_id: str = None, nickname: str = None):
        self.attach(MessageArray(File(file)), user_id, nickname)

    def attach_viedo(self, video: str, user_id: str = None, nickname: str = None):
        self.attach(MessageArray(Video(video)), user_id, nickname)

    def attach_forward(
        self, forward: Forward, user_id: str = None, nickname: str = None
    ):
        if forward.content is None:
            raise ValueError("Forward 对象的 content 不能为空")
        self.attach(MessageArray(forward), user_id, nickname)

    def attach_message_id(
        self, message_id: str, user_id: str = None, nickname: str = None
    ):
        event = run_coroutine(status.global_api.get_msg, message_id)
        user_id = user_id if user_id else event.user_id
        nickname = nickname if nickname else event.sender.nickname
        self.attach(event.message, user_id, nickname)

    def to_forward(self) -> Forward:
        return Forward(content=self.content)



class Sentence(str):
    """继承自 str 的句子类型，用于参数解析时捕获完整句子（包含空格）

    与普通 str 的区别：
    - str: 在参数解析时按空格分割，每次只取一个单词
    - Sentence: 在参数解析时取整个 Text 消息段的完整内容

    由于继承自 str，支持所有字符串操作
    """

    def __new__(cls, text: str):
        return super().__new__(cls, text)

    def __repr__(self):
        return f"Sentence('{str(self)}')"



from .api import BotAPI
from .api_account import AccountAPI
from .api_group import GroupAPI
from .api_message import MessageAPI
from .api_private import PrivateAPI
from .api_support import SupportAPI
from .utils import (
    BaseAPI,
    APIReturnStatus,
    MessageAPIReturnStatus,
    NapCatAPIError,
    ExclusiveArgumentError,
    check_exclusive_argument,
)

__all__ = [
    "BotAPI",
    "AccountAPI",
    "GroupAPI",
    "MessageAPI",
    "PrivateAPI",
    "SupportAPI",
    "BaseAPI",
    "APIReturnStatus",
    "MessageAPIReturnStatus",
    "NapCatAPIError",
    "ExclusiveArgumentError",
    "check_exclusive_argument",
]



import time
import uuid
from typing import Union, Optional
from ncatbot.core.event import (
    GroupMessageEvent,
    PrivateMessageEvent,
    NoticeEvent,
    RequestEvent,
)
from ncatbot.core.event.message_segment import MessageArray, Text


class EventFactory:
    """事件工厂类，用于创建标准化的测试事件"""

    @staticmethod
    def _generate_message_id() -> str:
        """生成唯一消息 ID"""
        return str(int(time.time() * 1000000) + hash(str(uuid.uuid4())) % 10000)

    @staticmethod
    def _get_current_timestamp() -> int:
        """获取当前时间戳"""
        return int(time.time())

    @staticmethod
    def create_group_message(
        message: Union[str, MessageArray],
        group_id: str = "123456789",
        user_id: str = "987654321",
        nickname: str = "TestUser",
        card: Optional[str] = None,
        role: str = "member",
        self_id: str = "123456789",
        message_id: Optional[str] = None,
        **kwargs,
    ) -> GroupMessageEvent:
        """创建群聊消息事件

        Args:
            message: 消息内容，可以是字符串或 MessageArray
            group_id: 群号
            user_id: 发送者 QQ 号
            nickname: 发送者昵称
            card: 群昵称（可选）
            role: 群角色 ("member", "admin", "owner")
            self_id: 机器人 QQ 号
            message_id: 消息 ID（可选，自动生成）
        """
        if isinstance(message, str):
            message_array = MessageArray(Text(message))
            raw_message = message
        else:
            message_array = message
            raw_message = "".join(
                [
                    seg.text if hasattr(seg, "text") else f"[{seg.msg_seg_type}]"
                    for seg in message_array.messages
                ]
            )

        sender_data = {
            "user_id": user_id,
            "nickname": nickname,
            "card": card or nickname,
            "role": role,
        }

        event_data = {
            "post_type": "message",
            "message_type": "group",
            "sub_type": "normal",
            "message_id": message_id or EventFactory._generate_message_id(),
            "group_id": group_id,
            "user_id": user_id,
            "message": message_array.to_list(),
            "raw_message": raw_message,
            "sender": sender_data,
            "self_id": self_id,
            "time": EventFactory._get_current_timestamp(),
            **kwargs,
        }

        return GroupMessageEvent(event_data)

    @staticmethod
    def create_private_message(
        message: Union[str, MessageArray],
        user_id: str = "987654321",
        nickname: str = "TestUser",
        self_id: str = "123456789",
        message_id: Optional[str] = None,
        sub_type: str = "friend",
        **kwargs,
    ) -> PrivateMessageEvent:
        """创建私聊消息事件"""
        if isinstance(message, str):
            message_array = MessageArray(Text(message))
            raw_message = message
        else:
            message_array = message
            raw_message = "".join(
                [
                    seg.text if hasattr(seg, "text") else f"[{seg.msg_seg_type}]"
                    for seg in message_array.messages
                ]
            )

        sender_data = {"user_id": user_id, "nickname": nickname}

        event_data = {
            "post_type": "message",
            "message_type": "private",
            "sub_type": sub_type,
            "message_id": message_id or EventFactory._generate_message_id(),
            "user_id": user_id,
            "message": message_array.to_list(),
            "raw_message": raw_message,
            "sender": sender_data,
            "self_id": self_id,
            "time": EventFactory._get_current_timestamp(),
            **kwargs,
        }

        return PrivateMessageEvent(event_data)

    @staticmethod
    def create_notice_event(
        notice_type: str,
        user_id: str = "987654321",
        group_id: Optional[str] = None,
        self_id: str = "123456789",
        sub_type: Optional[str] = None,
        **kwargs,
    ) -> NoticeEvent:
        """创建通知事件"""
        event_data = {
            "post_type": "notice",
            "notice_type": notice_type,
            "user_id": user_id,
            "self_id": self_id,
            "time": EventFactory._get_current_timestamp(),
            **kwargs,
        }

        if group_id:
            event_data["group_id"] = group_id
        if sub_type:
            event_data["sub_type"] = sub_type

        return NoticeEvent(event_data)

    @staticmethod
    def create_request_event(
        request_type: str,
        user_id: str = "987654321",
        flag: str = "test_flag",
        self_id: str = "123456789",
        sub_type: Optional[str] = None,
        **kwargs,
    ) -> RequestEvent:
        """创建请求事件"""
        event_data = {
            "post_type": "request",
            "request_type": request_type,
            "user_id": user_id,
            "flag": flag,
            "self_id": self_id,
            "time": EventFactory._get_current_timestamp(),
            **kwargs,
        }

        if sub_type:
            event_data["sub_type"] = sub_type

        return RequestEvent(event_data)



"""命令系统异常定义

分层的异常系统，提供精确的错误信息和智能修正建议。
"""

from typing import Optional, List, Any, Dict
from dataclasses import dataclass


class CommandSystemError(Exception):
    """命令系统基础异常"""

    def __init__(
        self,
        message: str,
        details: Optional[str] = None,
        suggestions: Optional[List[str]] = None,
    ):
        super().__init__(message)
        self.message = message
        self.details = details or ""
        self.suggestions = suggestions or []

    def __str__(self) -> str:
        result = self.message
        if self.details:
            result += f"\n详细信息: {self.details}"
        if self.suggestions:
            result += f"\n建议: {self.suggestions if isinstance(self.suggestions, str) else '; '.join(self.suggestions)}"
        return result


class CommandRegistrationError(CommandSystemError):
    """命令注册时错误 - 给开发者看的"""

    def __init__(self, command_name: str, message: str, **kwargs):
        self.command_name = command_name
        super().__init__(f"命令 '{command_name}' 注册失败: {message}", **kwargs)


class ParameterError(CommandSystemError):
    """参数定义错误"""

    def __init__(self, param_name: str, message: str, **kwargs):
        self.param_name = param_name
        super().__init__(f"参数 '{param_name}' 定义错误: {message}", **kwargs)


class ValidationError(CommandSystemError):
    """参数验证错误"""

    def __init__(self, param_name: str, value: Any, expected_type: str, **kwargs):
        self.param_name = param_name
        self.value = value
        self.expected_type = expected_type
        super().__init__(f"参数 '{param_name}' 验证失败", **kwargs)


class ArgumentError(CommandSystemError):
    """命令参数错误 - 给用户看的"""

    def __init__(self, command_name: str, message: str, **kwargs):
        self.command_name = command_name
        super().__init__(f"命令 '{command_name}' 参数错误: {message}", **kwargs)


class OptionError(CommandSystemError):
    """选项错误"""

    def __init__(self, option_name: str, message: str, **kwargs):
        self.option_name = option_name
        super().__init__(f"选项 '{option_name}' 错误: {message}", **kwargs)


class CommandNotFoundError(CommandSystemError):
    """命令不存在错误"""

    def __init__(self, command_name: str, available_commands: List[str], **kwargs):
        self.command_name = command_name
        self.available_commands = available_commands

        # 生成相似命令建议
        similar_commands = self._find_similar_commands(command_name, available_commands)
        suggestions = []
        if similar_commands:
            suggestions.append(f"你可能想要: {', '.join(similar_commands[:3])}")
        suggestions.append("输入 /help 查看所有可用命令")

        super().__init__(
            f"未知命令 '{command_name}'",
            details=f"可用命令: {', '.join(available_commands[:5])}{'...' if len(available_commands) > 5 else ''}",
            suggestions=suggestions,
            **kwargs,
        )

    def _find_similar_commands(self, target: str, commands: List[str]) -> List[str]:
        """查找相似的命令名"""
        import difflib

        return difflib.get_close_matches(target, commands, n=3, cutoff=0.6)


class TypeConversionError(ValidationError):
    """类型转换错误"""

    def __init__(
        self,
        param_name: str,
        value: Any,
        target_type: str,
        conversion_errors: List[str],
        **kwargs,
    ):
        self.conversion_errors = conversion_errors

        details = (
            f"值 '{value}' (类型: {type(value).__name__}) 无法转换为 {target_type}"
        )
        if conversion_errors:
            details += f"\n转换失败原因: {'; '.join(conversion_errors)}"

        super().__init__(param_name, value, target_type, details=details, **kwargs)


class MultiTypeConversionError(TypeConversionError):
    """多类型转换错误"""

    def __init__(
        self,
        param_name: str,
        value: Any,
        supported_types: List[str],
        type_errors: Dict[str, str],
        type_hints: Dict[str, str] = None,
        **kwargs,
    ):
        self.supported_types = supported_types
        self.type_errors = type_errors
        self.type_hints = type_hints or {}

        # 构建详细的错误信息
        details = f"值 '{value}' 无法转换为任何支持的类型\n"
        details += f"支持的类型: {', '.join(supported_types)}\n\n"

        details += "各类型转换失败原因:\n"
        for type_name, error in type_errors.items():
            details += f"  • {type_name}: {error}\n"

        if self.type_hints:
            details += "\n类型说明:\n"
            for type_name, hint in self.type_hints.items():
                details += f"  • {type_name}: {hint}\n"

        suggestions = ["请检查输入格式是否正确", "查看命令帮助了解正确的参数格式"]

        # 先调用父类构造函数
        super().__init__(
            param_name,
            value,
            f"({' 或 '.join(supported_types)})",
            list(type_errors.values()),
            **kwargs,
        )

        # 然后覆盖详细信息和建议
        if "details" not in kwargs:
            self.details = details
        if "suggestions" not in kwargs:
            self.suggestions = suggestions


class MutuallyExclusiveError(OptionError):
    """互斥选项错误"""

    def __init__(self, conflicting_options: List[str], **kwargs):
        self.conflicting_options = conflicting_options
        options_str = " 和 ".join(conflicting_options)
        super().__init__(
            options_str,
            f"选项 {options_str} 不能同时使用",
            suggestions=[f"请只选择其中一个: {', '.join(conflicting_options)}"],
            **kwargs,
        )


class MissingRequiredParameterError(ArgumentError):
    """缺少必需参数错误"""

    def __init__(self, command_name: str, param_name: str, param_type: str, **kwargs):
        self.param_name = param_name
        self.param_type = param_type

        super().__init__(
            command_name,
            f"缺少必需参数 '{param_name}'",
            details=f"参数类型: {param_type}",
            suggestions=[
                f"请提供 {param_name} 参数",
                f"输入 /{command_name} --help 查看详细用法",
            ],
            **kwargs,
        )


class TooManyArgumentsError(ArgumentError):
    """参数过多错误"""

    def __init__(
        self, command_name: str, expected_count: int, actual_count: int, **kwargs
    ):
        self.expected_count = expected_count
        self.actual_count = actual_count

        super().__init__(
            command_name,
            "参数过多",
            details=f"期望 {expected_count} 个参数，实际收到 {actual_count} 个",
            suggestions=[
                "检查是否有多余的参数",
                "使用引号包围包含空格的参数",
                f"输入 /{command_name} --help 查看正确用法",
            ],
            **kwargs,
        )


@dataclass
class ErrorContext:
    """错误上下文信息"""

    command_name: str
    input_text: str
    current_position: int
    available_commands: List[str]
    similar_commands: List[str]
    expected_parameters: List[str]
    provided_parameters: List[str]


class ErrorHandler:
    """统一错误处理器"""

    def __init__(self):
        self.error_formatters = {
            CommandNotFoundError: self._format_command_not_found,
            ArgumentError: self._format_argument_error,
            ValidationError: self._format_validation_error,
            MultiTypeConversionError: self._format_multi_type_error,
        }

    def format_error(
        self, error: CommandSystemError, context: Optional[ErrorContext] = None
    ) -> str:
        """格式化错误信息为用户友好的格式"""
        formatter = self.error_formatters.get(type(error), self._format_default)
        return formatter(error, context)

    def _format_command_not_found(
        self, error: CommandNotFoundError, context: Optional[ErrorContext]
    ) -> str:
        """格式化命令不存在错误"""
        msg = f"❌ {error.message}\n"

        if error.available_commands:
            # 显示相似命令
            similar = [
                cmd
                for cmd in error.available_commands
                if self._calculate_similarity(error.command_name, cmd) > 0.5
            ][:3]
            if similar:
                msg += f"💡 你可能想要: {', '.join(similar)}\n"

            # 显示部分可用命令
            msg += f"📋 可用命令: {', '.join(error.available_commands[:5])}"
            if len(error.available_commands) > 5:
                msg += "..."
            msg += "\n"

        msg += "❓ 输入 /help 查看所有命令"
        return msg

    def _format_argument_error(
        self, error: ArgumentError, context: Optional[ErrorContext]
    ) -> str:
        """格式化参数错误"""
        msg = f"❌ {error.message}\n"
        if error.details:
            msg += f"📝 {error.details}\n"
        if error.suggestions:
            msg += f"💡 建议: {'; '.join(error.suggestions)}"
        return msg

    def _format_validation_error(
        self, error: ValidationError, context: Optional[ErrorContext]
    ) -> str:
        """格式化验证错误"""
        msg = "❌ 参数验证失败\n"
        msg += f"📝 参数: {error.param_name}\n"
        msg += f"📝 输入值: {error.value}\n"
        msg += f"📝 期望类型: {error.expected_type}\n"
        if error.suggestions:
            msg += f"💡 {'; '.join(error.suggestions)}"
        return msg

    def _format_multi_type_error(
        self, error: MultiTypeConversionError, context: Optional[ErrorContext]
    ) -> str:
        """格式化多类型转换错误"""
        msg = f"❌ 参数 '{error.param_name}' 类型错误\n\n"
        msg += f"📝 您的输入: {error.value} ({type(error.value).__name__})\n\n"

        msg += "✅ 支持的类型:\n"
        for i, type_name in enumerate(error.supported_types, 1):
            msg += f"  {i}. {type_name}"
            if type_name in error.type_hints:
                msg += f" - {error.type_hints[type_name]}"
            msg += "\n"

        if error.suggestions:
            msg += f"\n💡 {'; '.join(error.suggestions)}"

        return msg

    def _format_default(
        self, error: CommandSystemError, context: Optional[ErrorContext]
    ) -> str:
        """默认错误格式化"""
        return f"❌ {error.message}"

    def _calculate_similarity(self, a: str, b: str) -> float:
        """计算字符串相似度"""
        import difflib

        return difflib.SequenceMatcher(None, a.lower(), b.lower()).ratio()



"""
NcatBot 测试工具模块

提供用于插件测试的各种工具和辅助类
"""

from .event_factory import EventFactory
from .client_mixin import ClientMixin
from .test_helper import TestHelper
from .mock_api import MockAPIAdapter
from .test_client import TestClient

__all__ = ["EventFactory", "ClientMixin", "TestHelper", "MockAPIAdapter", "TestClient"]



"""字符串分词器测试

测试 StringTokenizer 的所有功能，包括：
- 基本分词
- 短选项和长选项识别
- 参数赋值
- 转义字符串处理
- 错误处理
"""

from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system.lexer import (
    StringTokenizer,
    TokenType,
    QuoteMismatchError,
    InvalidEscapeSequenceError,
)


class TestStringTokenizer:
    """字符串分词器测试类"""

    def test_basic_words(self):
        """测试基本单词分词"""
        tokenizer = StringTokenizer("hello world test")
        tokens = tokenizer.tokenize()

        # 应该有 3 个 WORD token + 1 个 EOF token
        assert len(tokens) == 4
        assert tokens[0].type == TokenType.WORD
        assert tokens[0].value == "hello"
        assert tokens[1].type == TokenType.WORD
        assert tokens[1].value == "world"
        assert tokens[2].type == TokenType.WORD
        assert tokens[2].value == "test"
        assert tokens[3].type == TokenType.EOF

    def test_short_options(self):
        """测试短选项识别"""
        # 单个短选项
        tokenizer = StringTokenizer("-v")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 2  # option + EOF
        assert tokens[0].type == TokenType.SHORT_OPTION
        assert tokens[0].value == "v"

        # 组合短选项
        tokenizer = StringTokenizer("-xvf")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 2  # option + EOF
        assert tokens[0].type == TokenType.SHORT_OPTION
        assert tokens[0].value == "xvf"

        # 多个短选项
        tokenizer = StringTokenizer("-v -x -f")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 4  # 3 options + EOF
        assert all(t.type == TokenType.SHORT_OPTION for t in tokens[:-1])
        assert [t.value for t in tokens[:-1]] == ["v", "x", "f"]

    def test_long_options(self):
        """测试长选项识别"""
        # 单个长选项
        tokenizer = StringTokenizer("--verbose")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 2  # option + EOF
        assert tokens[0].type == TokenType.LONG_OPTION
        assert tokens[0].value == "verbose"

        # 多个长选项
        tokenizer = StringTokenizer("--verbose --debug --help")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 4  # 3 options + EOF
        assert all(t.type == TokenType.LONG_OPTION for t in tokens[:-1])
        assert [t.value for t in tokens[:-1]] == ["verbose", "debug", "help"]

    def test_option_assignments(self):
        """测试选项赋值"""
        # 短选项赋值
        tokenizer = StringTokenizer("-p=8080")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 4  # option + separator + value + EOF
        assert tokens[0].type == TokenType.SHORT_OPTION
        assert tokens[0].value == "p"
        assert tokens[1].type == TokenType.SEPARATOR
        assert tokens[1].value == "="
        assert tokens[2].type == TokenType.WORD
        assert tokens[2].value == "8080"

        # 长选项赋值
        tokenizer = StringTokenizer("--config=app.json")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 4  # option + separator + value + EOF
        assert tokens[0].type == TokenType.LONG_OPTION
        assert tokens[0].value == "config"
        assert tokens[1].type == TokenType.SEPARATOR
        assert tokens[1].value == "="
        assert tokens[2].type == TokenType.WORD
        assert tokens[2].value == "app.json"

        # 引用字符串赋值
        tokenizer = StringTokenizer('--message="hello world"')
        tokens = tokenizer.tokenize()
        assert len(tokens) == 4  # option + separator + value + EOF
        assert tokens[0].type == TokenType.LONG_OPTION
        assert tokens[0].value == "message"
        assert tokens[1].type == TokenType.SEPARATOR
        assert tokens[1].value == "="
        assert tokens[2].type == TokenType.QUOTED_STRING
        assert tokens[2].value == "hello world"

    def test_quoted_strings(self):
        """测试引用字符串处理"""
        # 基本引用字符串
        tokenizer = StringTokenizer('"hello world"')
        tokens = tokenizer.tokenize()
        assert len(tokens) == 2  # string + EOF
        assert tokens[0].type == TokenType.QUOTED_STRING
        assert tokens[0].value == "hello world"

        # 空引用字符串
        tokenizer = StringTokenizer('""')
        tokens = tokenizer.tokenize()
        assert len(tokens) == 2  # string + EOF
        assert tokens[0].type == TokenType.QUOTED_STRING
        assert tokens[0].value == ""

        # 多个引用字符串
        tokenizer = StringTokenizer('"first" "second" "third"')
        tokens = tokenizer.tokenize()
        assert len(tokens) == 4  # 3 strings + EOF
        assert all(t.type == TokenType.QUOTED_STRING for t in tokens[:-1])
        assert [t.value for t in tokens[:-1]] == ["first", "second", "third"]

    def test_escape_sequences(self):
        """测试转义序列"""
        # 基本转义序列
        escape_tests = [
            (r'"hello \"world\""', 'hello "world"'),
            (r'"line1\nline2"', "line1\nline2"),
            (r'"tab\there"', "tab\there"),
            (r'"carriage\rreturn"', "carriage\rreturn"),
            (r'"backslash\\"', "backslash\\"),
            (r'"slash\/"', "slash/"),
        ]

        for input_str, expected in escape_tests:
            tokenizer = StringTokenizer(input_str)
            tokens = tokenizer.tokenize()
            assert len(tokens) == 2  # string + EOF
            assert tokens[0].type == TokenType.QUOTED_STRING
            assert tokens[0].value == expected, f"Failed for input: {input_str}"

    def test_whitespace_handling(self):
        """测试空白字符处理"""
        # 多个空格
        tokenizer = StringTokenizer("  hello   world  ")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 3  # 2 words + EOF
        assert [t.value for t in tokens[:-1]] == ["hello", "world"]

        # 制表符和换行符
        tokenizer = StringTokenizer("\thello\nworld\r\n")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 3  # 2 words + EOF
        assert [t.value for t in tokens[:-1]] == ["hello", "world"]

        # 纯空白字符
        tokenizer = StringTokenizer("   \t\n\r  ")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 1  # 只有 EOF
        assert tokens[0].type == TokenType.EOF

    def test_complex_commands(self):
        """测试复杂命令解析"""
        complex_commands = [
            'deploy --env=prod -xvf "my app" --debug',
            'backup "source dir" --dest="/backup/folder" -c=gzip',
            "docker run --name=myapp -p=8080:80 -d nginx:latest",
            'git commit -m "Initial commit" --author="张三 <zhangsan@example.com>"',
        ]

        for command in complex_commands:
            tokenizer = StringTokenizer(command)
            tokens = tokenizer.tokenize()

            # 验证最后一个是 EOF token
            assert tokens[-1].type == TokenType.EOF

            # 验证没有空的 token 值（除了 EOF）
            for token in tokens[:-1]:
                assert token.value != ""

            # 验证位置信息正确
            for token in tokens:
                assert token.position >= 0

    def test_error_handling(self):
        """测试错误处理"""
        # # 未闭合的引号
        # with pytest.raises(QuoteMismatchError) as exc_info:
        #     tokenizer = StringTokenizer('"unclosed quote')
        #     tokenizer.tokenize()
        # assert "Unmatched quote" in str(exc_info.value)

        # # 无效转义序列
        # with pytest.raises(InvalidEscapeSequenceError) as exc_info:
        #     tokenizer = StringTokenizer(r'"invalid \z escape"')
        #     tokenizer.tokenize()
        # assert "Invalid escape sequence" in str(exc_info.value)

        # # 转义序列在字符串末尾
        # with pytest.raises(InvalidEscapeSequenceError) as exc_info:
        #     tokenizer = StringTokenizer(r'"ends with backslash \"')
        #     tokenizer.tokenize()
        # assert "Invalid escape sequence" in str(exc_info.value)

    def test_edge_cases(self):
        """测试边界情况"""
        # 空字符串
        tokenizer = StringTokenizer("")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 1
        assert tokens[0].type == TokenType.EOF

        # 只有选项分隔符
        tokenizer = StringTokenizer("=")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 2  # separator + EOF
        assert tokens[0].type == TokenType.SEPARATOR
        assert tokens[0].value == "="

        # 只有短横线
        tokenizer = StringTokenizer("-")
        tokens = tokenizer.tokenize()
        assert len(tokens) == 2  # word + EOF
        assert tokens[0].type == TokenType.WORD
        assert tokens[0].value == "-"

        # 只有双短横线 - 这会触发空长选项错误
        try:
            tokenizer = StringTokenizer("--")
            tokens = tokenizer.tokenize()
            # 如果没有抛出异常，检查是否被当作普通词处理
            assert len(tokens) == 2  # word + EOF
            assert tokens[0].type == TokenType.WORD
            assert tokens[0].value == "--"
        except Exception:
            # 空长选项会抛出异常，这是预期的
            pass

    def test_position_tracking(self):
        """测试位置跟踪"""
        tokenizer = StringTokenizer("hello --world")
        tokens = tokenizer.tokenize()

        # 检查位置信息
        assert tokens[0].position == 0  # "hello" 开始位置
        assert tokens[1].position == 8  # "--world" 开始位置

        # 复杂示例
        tokenizer = StringTokenizer('test "quoted" -v')
        tokens = tokenizer.tokenize()

        assert tokens[0].position == 0  # "test"
        assert tokens[1].position == 7  # "quoted"
        assert tokens[2].position == 15  # "-v"


def test_integration_with_imports():
    """测试导入和集成"""
    # 测试所有必要的导入都可用
    from ncatbot.plugin_system.builtin_plugin.unified_registry.command_system.lexer import (
        StringTokenizer,
        TokenType,
    )

    # 基本功能测试
    tokenizer = StringTokenizer('-v --config="test.json" hello')
    tokens = tokenizer.tokenize()

    assert len(tokens) == 6  # option + option + separator + string + word + EOF
    assert tokens[0].type == TokenType.SHORT_OPTION
    assert tokens[1].type == TokenType.LONG_OPTION
    assert tokens[2].type == TokenType.SEPARATOR
    assert tokens[3].type == TokenType.QUOTED_STRING
    assert tokens[4].type == TokenType.WORD


if __name__ == "__main__":
    print("运行字符串分词器测试...")

    # 创建测试实例
    test_instance = TestStringTokenizer()

    # 运行所有测试方法
    test_methods = [
        ("基本单词分词", test_instance.test_basic_words),
        ("短选项识别", test_instance.test_short_options),
        ("长选项识别", test_instance.test_long_options),
        ("选项赋值", test_instance.test_option_assignments),
        ("引用字符串", test_instance.test_quoted_strings),
        ("转义序列", test_instance.test_escape_sequences),
        ("空白字符处理", test_instance.test_whitespace_handling),
        ("复杂命令", test_instance.test_complex_commands),
        ("边界情况", test_instance.test_edge_cases),
        ("位置跟踪", test_instance.test_position_tracking),
    ]

    for test_name, test_method in test_methods:
        try:
            test_method()
            print(f"✓ {test_name}测试通过")
        except Exception as e:
            print(f"✗ {test_name}测试失败: {e}")

    # 错误处理测试（需要特殊处理）
    print("\n测试错误处理...")
    try:
        StringTokenizer('"未闭合引号').tokenize()
    except QuoteMismatchError:
        print("✓ 引号不匹配错误捕获成功")

    try:
        StringTokenizer(r'"无效\z转义"').tokenize()
    except InvalidEscapeSequenceError:
        print("✓ 无效转义序列错误捕获成功")

    # 集成测试
    test_integration_with_imports()
    print("✓ 导入和集成测试通过")

    print("\n字符串分词器所有测试完成！✨")
